/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, uhd, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_uhd_usrp_block = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_subdev_spec = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_subdev_spec = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_num_mboards = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_samp_rate = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_samp_rate = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_samp_rates = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_center_freq_0 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_center_freq_1 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_center_freq = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_freq_range = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_gain_0 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_gain_1 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_normalized_gain = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gain_0 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gain_1 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_normalized_gain = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gain_names = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gain_range_0 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gain_range_1 = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_has_power_reference = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_power_reference = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_power_reference = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_power_range = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_antenna = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_antenna = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_antennas = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_bandwidth = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_bandwidth = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_bandwidth_range = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_sensor = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_sensor_names = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_dboard_sensor = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_dboard_sensor_names = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_mboard_sensor = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_mboard_sensor_names = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_time_source = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_time_sources = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_clock_source = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_clock_source = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_clock_sources = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_clock_rate = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_clock_rate = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_time_now = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_time_last_pps = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_time_now = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_time_next_pps = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_time_unknown_pps = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_command_time = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_clear_command_time = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_dboard_iface = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_device = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_user_register = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_time_source = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_stream_args = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gpio_banks = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_gpio_attr = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_gpio_attr = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_filter_names = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_set_filter = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_filter = R"doc()doc";


static const char* __doc_gr_uhd_usrp_block_get_usrp_info = R"doc()doc";


static const char* __doc_gr_uhd_cmd_chan_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_gain_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_power_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_freq_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_lo_offset_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_tune_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_lo_freq_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_dsp_freq_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_rate_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_bandwidth_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_time_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_mboard_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_antenna_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_direction_key = R"doc()doc";


static const char* __doc_gr_uhd_cmd_tag_key = R"doc()doc";


static const char* __doc_gr_uhd_direction_rx = R"doc()doc";


static const char* __doc_gr_uhd_direction_tx = R"doc()doc";
