/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, filter, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff = R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_fff_fir_filter_with_buffer_fff_0 =
        R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_fff_fir_filter_with_buffer_fff_1 =
        R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_filter_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_filter_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_filterN =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_filterNdec =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_ntaps = R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_set_taps =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_fff_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc = R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_fir_filter_with_buffer_ccc_0 =
        R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_fir_filter_with_buffer_ccc_1 =
        R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_filter_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_filter_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_filterN =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_filterNdec =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_ntaps = R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_set_taps =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccc_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf = R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_fir_filter_with_buffer_ccf_0 =
        R"doc()doc";


static const char*
    __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_fir_filter_with_buffer_ccf_1 =
        R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_filter_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_filter_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_filterN =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_filterNdec =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_ntaps = R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_set_taps =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_fir_filter_with_buffer_ccf_taps = R"doc()doc";
