/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(correlate_access_code_bb.h) */
/* BINDTOOL_HEADER_FILE_HASH(ce5720570bb5308c29ea04e126c65147)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/correlate_access_code_bb.h>
// pydoc.h is automatically generated in the build directory
#include <correlate_access_code_bb_pydoc.h>

void bind_correlate_access_code_bb(py::module& m)
{

    using correlate_access_code_bb = ::gr::digital::correlate_access_code_bb;


    py::class_<correlate_access_code_bb,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<correlate_access_code_bb>>(
        m, "correlate_access_code_bb", D(correlate_access_code_bb))

        .def(py::init(&correlate_access_code_bb::make),
             py::arg("access_code"),
             py::arg("threshold"),
             D(correlate_access_code_bb, make))


        .def("set_access_code",
             &correlate_access_code_bb::set_access_code,
             py::arg("access_code"),
             D(correlate_access_code_bb, set_access_code))

        ;
}
