/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, digital, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_digital_symbol_sync_ff = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_symbol_sync_ff_0 = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_symbol_sync_ff_1 = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_make = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_damping_factor = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_ted_gain = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_alpha = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_beta = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_sps = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_damping_factor = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_ted_gain = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_alpha = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_beta = R"doc()doc";


static const char* __doc_gr_digital_symbol_sync_ff_set_sps = R"doc()doc";
