/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, blocks, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_blocks_control_loop = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_control_loop_0 = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_control_loop_1 = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_control_loop_2 = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_update_gains = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_advance_loop = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_phase_wrap = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_frequency_limit = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_damping_factor = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_alpha = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_beta = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_frequency = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_phase = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_max_freq = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_set_min_freq = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_damping_factor = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_alpha = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_beta = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_frequency = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_phase = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_max_freq = R"doc()doc";


static const char* __doc_gr_blocks_control_loop_get_min_freq = R"doc()doc";


static const char* __doc_gr_blocks_tanhf_lut = R"doc()doc";
