/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(skiphead.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(7e569f2f33544fdc21963c5b4dc69972)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/skiphead.h>
// pydoc.h is automatically generated in the build directory
#include <skiphead_pydoc.h>

void bind_skiphead(py::module& m)
{

    using skiphead = ::gr::blocks::skiphead;


    py::class_<skiphead, gr::block, gr::basic_block, std::shared_ptr<skiphead>>(
        m, "skiphead", D(skiphead))

        .def(py::init(&skiphead::make),
             py::arg("itemsize"),
             py::arg("nitems_to_skip"),
             D(skiphead, make))


        ;
}
