/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_prefs = R"doc()doc";


static const char* __doc_gr_prefs_prefs = R"doc()doc";


static const char* __doc_gr_prefs_singleton = R"doc()doc";


static const char* __doc_gr_prefs_add_config_file = R"doc()doc";


static const char* __doc_gr_prefs_to_string = R"doc()doc";


static const char* __doc_gr_prefs_save = R"doc()doc";


static const char* __doc_gr_prefs_has_section = R"doc()doc";


static const char* __doc_gr_prefs_has_option = R"doc()doc";


static const char* __doc_gr_prefs_get_string = R"doc()doc";


static const char* __doc_gr_prefs_set_string = R"doc()doc";


static const char* __doc_gr_prefs_get_bool = R"doc()doc";


static const char* __doc_gr_prefs_set_bool = R"doc()doc";


static const char* __doc_gr_prefs_get_long = R"doc()doc";


static const char* __doc_gr_prefs_set_long = R"doc()doc";


static const char* __doc_gr_prefs_get_double = R"doc()doc";


static const char* __doc_gr_prefs_set_double = R"doc()doc";
