/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(framer_sink_1.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(c4d313da318fada32f4b7cf6de0068b8)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/framer_sink_1.h>
// pydoc.h is automatically generated in the build directory
#include <framer_sink_1_pydoc.h>

void bind_framer_sink_1(py::module& m)
{

    using framer_sink_1 = ::gr::digital::framer_sink_1;


    py::class_<framer_sink_1,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<framer_sink_1>>(m, "framer_sink_1", D(framer_sink_1))

        .def(py::init(&framer_sink_1::make),
             py::arg("target_queue"),
             D(framer_sink_1, make))


        ;
}
