/*
 * Copyright 2022 Marcus Müller
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(dictionary_logger_backend.h)                               */
/* BINDTOOL_HEADER_FILE_HASH(6f741a4086529299ba597288b7277b87)                     */
/***********************************************************************************/

#include <pybind11/chrono.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include <gnuradio/dictionary_logger_backend.h>
#include <dictionary_logger_backend_pydoc.h>
#include <memory>


namespace py = pybind11;
void bind_dictionary_logger_backend(py::module& m)
{

    using dictionary_logger_backend = gr::dictionary_logger_backend;

    py::class_<spdlog::sinks::sink, std::shared_ptr<spdlog::sinks::sink>>(m, "sink");

    py::class_<dictionary_logger_backend,
               spdlog::sinks::sink,
               std::shared_ptr<dictionary_logger_backend>>(
        m, "dictionary_logger_backend", D(dictionary_logger_backend))

        .def(py::init(), D(dictionary_logger_backend, dictionary_logger_backend))
        .def(py::init<std::regex>(),
             D(dictionary_logger_backend, dictionary_logger_backend))

        .def("get_map",
             &dictionary_logger_backend::get_map,
             D(dictionary_logger_backend, get_map))

        ;
}
