/*
 * Copyright 2020-2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(count_bits.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(b999075adec115941653f24e8e41f18b)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/count_bits.h>
// pydoc.h is automatically generated in the build directory
#include <count_bits_pydoc.h>

void bind_count_bits(py::module& m)
{


    m.def("count_bits8", &::gr::blocks::count_bits8, py::arg("x"), D(count_bits8));


    m.def("count_bits16", &::gr::blocks::count_bits16, py::arg("x"), D(count_bits16));


    m.def("count_bits32", &::gr::blocks::count_bits32, py::arg("x"), D(count_bits32));


    m.def("count_bits64", &::gr::blocks::count_bits64, py::arg("x"), D(count_bits64));
}
