/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(filterbank_vcvcf.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(98bfd56543cc1e94c2080d7fa089084e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/filterbank_vcvcf.h>
// pydoc.h is automatically generated in the build directory
#include <filterbank_vcvcf_pydoc.h>

void bind_filterbank_vcvcf(py::module& m)
{
    using filterbank_vcvcf = gr::filter::filterbank_vcvcf;


    py::class_<filterbank_vcvcf,
               gr::block,
               gr::basic_block,
               std::shared_ptr<filterbank_vcvcf>>(
        m, "filterbank_vcvcf", D(filterbank_vcvcf))

        .def(
            py::init(&filterbank_vcvcf::make), py::arg("taps"), D(filterbank_vcvcf, make))


        .def("set_taps",
             &filterbank_vcvcf::set_taps,
             py::arg("taps"),
             D(filterbank_vcvcf, set_taps))


        .def("print_taps", &filterbank_vcvcf::print_taps, D(filterbank_vcvcf, print_taps))


        .def("taps", &filterbank_vcvcf::taps, D(filterbank_vcvcf, taps))

        ;
}
