/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(encode_ccsds_27_bb.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(80958c26294b511baa284af535cd1e7e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/encode_ccsds_27_bb.h>
// pydoc.h is automatically generated in the build directory
#include <encode_ccsds_27_bb_pydoc.h>

void bind_encode_ccsds_27_bb(py::module& m)
{

    using encode_ccsds_27_bb = ::gr::fec::encode_ccsds_27_bb;


    py::class_<encode_ccsds_27_bb,
               gr::sync_interpolator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<encode_ccsds_27_bb>>(
        m, "encode_ccsds_27_bb", D(encode_ccsds_27_bb))

        .def(py::init(&encode_ccsds_27_bb::make), D(encode_ccsds_27_bb, make))


        ;
}
