/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(pmt, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_pmt_pmt_base = R"doc()doc";


static const char* __doc_pmt_pmt_base_pmt_base = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_bool = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_symbol = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_number = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_integer = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_uint64 = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_real = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_complex = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_null = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_pair = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_tuple = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_dict = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_any = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_uniform_vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_u8vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_s8vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_u16vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_s16vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_u32vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_s32vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_u64vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_s64vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_f32vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_f64vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_c32vector = R"doc()doc";


static const char* __doc_pmt_pmt_base_is_c64vector = R"doc()doc";


static const char* __doc_pmt_exception = R"doc()doc";


static const char* __doc_pmt_exception_exception_0 = R"doc()doc";


static const char* __doc_pmt_exception_exception_1 = R"doc()doc";


static const char* __doc_pmt_wrong_type = R"doc()doc";


static const char* __doc_pmt_wrong_type_wrong_type_0 = R"doc()doc";


static const char* __doc_pmt_wrong_type_wrong_type_1 = R"doc()doc";


static const char* __doc_pmt_out_of_range = R"doc()doc";


static const char* __doc_pmt_out_of_range_out_of_range_0 = R"doc()doc";


static const char* __doc_pmt_out_of_range_out_of_range_1 = R"doc()doc";


static const char* __doc_pmt_notimplemented = R"doc()doc";


static const char* __doc_pmt_notimplemented_notimplemented_0 = R"doc()doc";


static const char* __doc_pmt_notimplemented_notimplemented_1 = R"doc()doc";


static const char* __doc_pmt_comparator = R"doc()doc";


static const char* __doc_pmt_comparator_comparator_0 = R"doc()doc";


static const char* __doc_pmt_comparator_comparator_1 = R"doc()doc";


static const char* __doc_pmt_get_PMT_NIL = R"doc()doc";


static const char* __doc_pmt_get_PMT_T = R"doc()doc";


static const char* __doc_pmt_get_PMT_F = R"doc()doc";


static const char* __doc_pmt_get_PMT_EOF = R"doc()doc";


static const char* __doc_pmt_is_bool = R"doc()doc";


static const char* __doc_pmt_is_true = R"doc()doc";


static const char* __doc_pmt_is_false = R"doc()doc";


static const char* __doc_pmt_from_bool = R"doc()doc";


static const char* __doc_pmt_to_bool = R"doc()doc";


static const char* __doc_pmt_is_symbol = R"doc()doc";


static const char* __doc_pmt_string_to_symbol = R"doc()doc";


static const char* __doc_pmt_intern = R"doc()doc";


static const char* __doc_pmt_symbol_to_string = R"doc()doc";


static const char* __doc_pmt_is_number = R"doc()doc";


static const char* __doc_pmt_is_integer = R"doc()doc";


static const char* __doc_pmt_from_long = R"doc()doc";


static const char* __doc_pmt_to_long = R"doc()doc";


static const char* __doc_pmt_is_uint64 = R"doc()doc";


static const char* __doc_pmt_from_uint64 = R"doc()doc";


static const char* __doc_pmt_to_uint64 = R"doc()doc";


static const char* __doc_pmt_is_real = R"doc()doc";


static const char* __doc_pmt_from_double = R"doc()doc";


static const char* __doc_pmt_from_float = R"doc()doc";


static const char* __doc_pmt_to_double = R"doc()doc";


static const char* __doc_pmt_to_float = R"doc()doc";


static const char* __doc_pmt_is_complex = R"doc()doc";


static const char* __doc_pmt_make_rectangular = R"doc()doc";


static const char* __doc_pmt_from_complex_0 = R"doc()doc";


static const char* __doc_pmt_from_complex_1 = R"doc()doc";


static const char* __doc_pmt_pmt_from_complex_0 = R"doc()doc";


static const char* __doc_pmt_pmt_from_complex_1 = R"doc()doc";


static const char* __doc_pmt_to_complex = R"doc()doc";


static const char* __doc_pmt_is_null = R"doc()doc";


static const char* __doc_pmt_is_pair = R"doc()doc";


static const char* __doc_pmt_cons = R"doc()doc";


static const char* __doc_pmt_dcons = R"doc()doc";


static const char* __doc_pmt_car = R"doc()doc";


static const char* __doc_pmt_cdr = R"doc()doc";


static const char* __doc_pmt_set_car = R"doc()doc";


static const char* __doc_pmt_set_cdr = R"doc()doc";


static const char* __doc_pmt_caar = R"doc()doc";


static const char* __doc_pmt_cadr = R"doc()doc";


static const char* __doc_pmt_cdar = R"doc()doc";


static const char* __doc_pmt_cddr = R"doc()doc";


static const char* __doc_pmt_caddr = R"doc()doc";


static const char* __doc_pmt_cadddr = R"doc()doc";


static const char* __doc_pmt_is_tuple = R"doc()doc";


static const char* __doc_pmt_make_tuple_0 = R"doc()doc";


static const char* __doc_pmt_make_tuple_1 = R"doc()doc";


static const char* __doc_pmt_make_tuple_2 = R"doc()doc";


static const char* __doc_pmt_make_tuple_3 = R"doc()doc";


static const char* __doc_pmt_make_tuple_4 = R"doc()doc";


static const char* __doc_pmt_make_tuple_5 = R"doc()doc";


static const char* __doc_pmt_make_tuple_6 = R"doc()doc";


static const char* __doc_pmt_make_tuple_7 = R"doc()doc";


static const char* __doc_pmt_make_tuple_8 = R"doc()doc";


static const char* __doc_pmt_make_tuple_9 = R"doc()doc";


static const char* __doc_pmt_make_tuple_10 = R"doc()doc";


static const char* __doc_pmt_to_tuple = R"doc()doc";


static const char* __doc_pmt_tuple_ref = R"doc()doc";


static const char* __doc_pmt_is_vector = R"doc()doc";


static const char* __doc_pmt_make_vector = R"doc()doc";


static const char* __doc_pmt_vector_ref = R"doc()doc";


static const char* __doc_pmt_vector_set = R"doc()doc";


static const char* __doc_pmt_vector_fill = R"doc()doc";


static const char* __doc_pmt_is_blob = R"doc()doc";


static const char* __doc_pmt_make_blob = R"doc()doc";


static const char* __doc_pmt_blob_data = R"doc()doc";


static const char* __doc_pmt_blob_length = R"doc()doc";


static const char* __doc_pmt_is_uniform_vector = R"doc()doc";


static const char* __doc_pmt_is_u8vector = R"doc()doc";


static const char* __doc_pmt_is_s8vector = R"doc()doc";


static const char* __doc_pmt_is_u16vector = R"doc()doc";


static const char* __doc_pmt_is_s16vector = R"doc()doc";


static const char* __doc_pmt_is_u32vector = R"doc()doc";


static const char* __doc_pmt_is_s32vector = R"doc()doc";


static const char* __doc_pmt_is_u64vector = R"doc()doc";


static const char* __doc_pmt_is_s64vector = R"doc()doc";


static const char* __doc_pmt_is_f32vector = R"doc()doc";


static const char* __doc_pmt_is_f64vector = R"doc()doc";


static const char* __doc_pmt_is_c32vector = R"doc()doc";


static const char* __doc_pmt_is_c64vector = R"doc()doc";


static const char* __doc_pmt_uniform_vector_itemsize = R"doc()doc";


static const char* __doc_pmt_make_u8vector = R"doc()doc";


static const char* __doc_pmt_make_s8vector = R"doc()doc";


static const char* __doc_pmt_make_u16vector = R"doc()doc";


static const char* __doc_pmt_make_s16vector = R"doc()doc";


static const char* __doc_pmt_make_u32vector = R"doc()doc";


static const char* __doc_pmt_make_s32vector = R"doc()doc";


static const char* __doc_pmt_make_u64vector = R"doc()doc";


static const char* __doc_pmt_make_s64vector = R"doc()doc";


static const char* __doc_pmt_make_f32vector = R"doc()doc";


static const char* __doc_pmt_make_f64vector = R"doc()doc";


static const char* __doc_pmt_make_c32vector = R"doc()doc";


static const char* __doc_pmt_make_c64vector = R"doc()doc";


static const char* __doc_pmt_init_u8vector_0 = R"doc()doc";


static const char* __doc_pmt_init_u8vector_1 = R"doc()doc";


static const char* __doc_pmt_init_s8vector_0 = R"doc()doc";


static const char* __doc_pmt_init_s8vector_1 = R"doc()doc";


static const char* __doc_pmt_init_u16vector_0 = R"doc()doc";


static const char* __doc_pmt_init_u16vector_1 = R"doc()doc";


static const char* __doc_pmt_init_s16vector_0 = R"doc()doc";


static const char* __doc_pmt_init_s16vector_1 = R"doc()doc";


static const char* __doc_pmt_init_u32vector_0 = R"doc()doc";


static const char* __doc_pmt_init_u32vector_1 = R"doc()doc";


static const char* __doc_pmt_init_s32vector_0 = R"doc()doc";


static const char* __doc_pmt_init_s32vector_1 = R"doc()doc";


static const char* __doc_pmt_init_u64vector_0 = R"doc()doc";


static const char* __doc_pmt_init_u64vector_1 = R"doc()doc";


static const char* __doc_pmt_init_s64vector_0 = R"doc()doc";


static const char* __doc_pmt_init_s64vector_1 = R"doc()doc";


static const char* __doc_pmt_init_f32vector_0 = R"doc()doc";


static const char* __doc_pmt_init_f32vector_1 = R"doc()doc";


static const char* __doc_pmt_init_f64vector_0 = R"doc()doc";


static const char* __doc_pmt_init_f64vector_1 = R"doc()doc";


static const char* __doc_pmt_init_c32vector_0 = R"doc()doc";


static const char* __doc_pmt_init_c32vector_1 = R"doc()doc";


static const char* __doc_pmt_init_c64vector_0 = R"doc()doc";


static const char* __doc_pmt_init_c64vector_1 = R"doc()doc";


static const char* __doc_pmt_u8vector_ref = R"doc()doc";


static const char* __doc_pmt_s8vector_ref = R"doc()doc";


static const char* __doc_pmt_u16vector_ref = R"doc()doc";


static const char* __doc_pmt_s16vector_ref = R"doc()doc";


static const char* __doc_pmt_u32vector_ref = R"doc()doc";


static const char* __doc_pmt_s32vector_ref = R"doc()doc";


static const char* __doc_pmt_u64vector_ref = R"doc()doc";


static const char* __doc_pmt_s64vector_ref = R"doc()doc";


static const char* __doc_pmt_f32vector_ref = R"doc()doc";


static const char* __doc_pmt_f64vector_ref = R"doc()doc";


static const char* __doc_pmt_c32vector_ref = R"doc()doc";


static const char* __doc_pmt_c64vector_ref = R"doc()doc";


static const char* __doc_pmt_u8vector_set = R"doc()doc";


static const char* __doc_pmt_s8vector_set = R"doc()doc";


static const char* __doc_pmt_u16vector_set = R"doc()doc";


static const char* __doc_pmt_s16vector_set = R"doc()doc";


static const char* __doc_pmt_u32vector_set = R"doc()doc";


static const char* __doc_pmt_s32vector_set = R"doc()doc";


static const char* __doc_pmt_u64vector_set = R"doc()doc";


static const char* __doc_pmt_s64vector_set = R"doc()doc";


static const char* __doc_pmt_f32vector_set = R"doc()doc";


static const char* __doc_pmt_f64vector_set = R"doc()doc";


static const char* __doc_pmt_c32vector_set = R"doc()doc";


static const char* __doc_pmt_c64vector_set = R"doc()doc";


static const char* __doc_pmt_uniform_vector_elements = R"doc()doc";


static const char* __doc_pmt_u8vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_s8vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_u16vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_s16vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_u32vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_s32vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_u64vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_s64vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_f32vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_f64vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_c32vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_c64vector_elements_0 = R"doc()doc";


static const char* __doc_pmt_u8vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_s8vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_u16vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_s16vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_u32vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_s32vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_u64vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_s64vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_f32vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_f64vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_c32vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_c64vector_elements_1 = R"doc()doc";


static const char* __doc_pmt_uniform_vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_u8vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_s8vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_u16vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_s16vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_u32vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_s32vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_u64vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_s64vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_f32vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_f64vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_c32vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_c64vector_writable_elements = R"doc()doc";


static const char* __doc_pmt_is_dict = R"doc()doc";


static const char* __doc_pmt_make_dict = R"doc()doc";


static const char* __doc_pmt_dict_add = R"doc()doc";


static const char* __doc_pmt_dict_delete = R"doc()doc";


static const char* __doc_pmt_dict_has_key = R"doc()doc";


static const char* __doc_pmt_dict_ref = R"doc()doc";


static const char* __doc_pmt_dict_items = R"doc()doc";


static const char* __doc_pmt_dict_keys = R"doc()doc";


static const char* __doc_pmt_dict_update = R"doc()doc";


static const char* __doc_pmt_dict_values = R"doc()doc";


static const char* __doc_pmt_is_any = R"doc()doc";


static const char* __doc_pmt_make_any = R"doc()doc";


static const char* __doc_pmt_any_ref = R"doc()doc";


static const char* __doc_pmt_any_set = R"doc()doc";


static const char* __doc_pmt_is_msg_accepter = R"doc()doc";


static const char* __doc_pmt_make_msg_accepter = R"doc()doc";


static const char* __doc_pmt_msg_accepter_ref = R"doc()doc";


static const char* __doc_pmt_is_pdu = R"doc()doc";


static const char* __doc_pmt_eq = R"doc()doc";


static const char* __doc_pmt_eqv = R"doc()doc";


static const char* __doc_pmt_equal = R"doc()doc";


static const char* __doc_pmt_assq = R"doc()doc";


static const char* __doc_pmt_assv = R"doc()doc";


static const char* __doc_pmt_assoc = R"doc()doc";


static const char* __doc_pmt_map = R"doc()doc";


static const char* __doc_pmt_reverse = R"doc()doc";


static const char* __doc_pmt_reverse_x = R"doc()doc";


static const char* __doc_pmt_acons = R"doc()doc";


static const char* __doc_pmt_nth = R"doc()doc";


static const char* __doc_pmt_nthcdr = R"doc()doc";


static const char* __doc_pmt_memq = R"doc()doc";


static const char* __doc_pmt_memv = R"doc()doc";


static const char* __doc_pmt_member = R"doc()doc";


static const char* __doc_pmt_subsetp = R"doc()doc";


static const char* __doc_pmt_list1 = R"doc()doc";


static const char* __doc_pmt_list2 = R"doc()doc";


static const char* __doc_pmt_list3 = R"doc()doc";


static const char* __doc_pmt_list4 = R"doc()doc";


static const char* __doc_pmt_list5 = R"doc()doc";


static const char* __doc_pmt_list6 = R"doc()doc";


static const char* __doc_pmt_list_add = R"doc()doc";


static const char* __doc_pmt_list_rm = R"doc()doc";


static const char* __doc_pmt_list_has = R"doc()doc";


static const char* __doc_pmt_is_eof_object = R"doc()doc";


static const char* __doc_pmt_read = R"doc()doc";


static const char* __doc_pmt_write = R"doc()doc";


static const char* __doc_pmt_write_string = R"doc()doc";


static const char* __doc_pmt_print = R"doc()doc";


static const char* __doc_pmt_serialize = R"doc()doc";


static const char* __doc_pmt_deserialize = R"doc()doc";


static const char* __doc_pmt_dump_sizeof = R"doc()doc";


static const char* __doc_pmt_serialize_str = R"doc()doc";


static const char* __doc_pmt_deserialize_str = R"doc()doc";
