/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(agc2_ff.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(5e1d2da6d6800f26f91c43aa7829628e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/analog/agc2_ff.h>
// pydoc.h is automatically generated in the build directory
#include <agc2_ff_pydoc.h>

void bind_agc2_ff(py::module& m)
{

    using agc2_ff = ::gr::analog::agc2_ff;


    py::class_<agc2_ff,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<agc2_ff>>(m, "agc2_ff", D(agc2_ff))

        .def(py::init(&agc2_ff::make),
             py::arg("attack_rate") = 0.10000000000000001,
             py::arg("decay_rate") = 0.01,
             py::arg("reference") = 1.,
             py::arg("gain") = 1.,
             py::arg("max_gain") = 0.,
             D(agc2_ff, make))


        .def("attack_rate", &agc2_ff::attack_rate, D(agc2_ff, attack_rate))


        .def("decay_rate", &agc2_ff::decay_rate, D(agc2_ff, decay_rate))


        .def("reference", &agc2_ff::reference, D(agc2_ff, reference))


        .def("gain", &agc2_ff::gain, D(agc2_ff, gain))


        .def("max_gain", &agc2_ff::max_gain, D(agc2_ff, max_gain))


        .def("set_attack_rate",
             &agc2_ff::set_attack_rate,
             py::arg("rate"),
             D(agc2_ff, set_attack_rate))


        .def("set_decay_rate",
             &agc2_ff::set_decay_rate,
             py::arg("rate"),
             D(agc2_ff, set_decay_rate))


        .def("set_reference",
             &agc2_ff::set_reference,
             py::arg("reference"),
             D(agc2_ff, set_reference))


        .def("set_gain", &agc2_ff::set_gain, py::arg("gain"), D(agc2_ff, set_gain))


        .def("set_max_gain",
             &agc2_ff::set_max_gain,
             py::arg("max_gain"),
             D(agc2_ff, set_max_gain))

        ;


    py::module m_kernel = m.def_submodule("kernel");
}
