/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_fast_cc_multiply = R"doc()doc";


static const char* __doc_gr_is_power_of_2 = R"doc()doc";


static const char* __doc_gr_fast_atan2f_0 = R"doc()doc";


static const char* __doc_gr_fast_atan2f_1 = R"doc()doc";


static const char* __doc_gr_branchless_clip = R"doc()doc";


static const char* __doc_gr_clip = R"doc()doc";


static const char* __doc_gr_binary_slicer = R"doc()doc";


static const char* __doc_gr_quad_45deg_slicer_0 = R"doc()doc";


static const char* __doc_gr_quad_0deg_slicer_0 = R"doc()doc";


static const char* __doc_gr_quad_45deg_slicer_1 = R"doc()doc";


static const char* __doc_gr_quad_0deg_slicer_1 = R"doc()doc";


static const char* __doc_gr_branchless_binary_slicer = R"doc()doc";


static const char* __doc_gr_branchless_quad_0deg_slicer_0 = R"doc()doc";


static const char* __doc_gr_branchless_quad_0deg_slicer_1 = R"doc()doc";


static const char* __doc_gr_branchless_quad_45deg_slicer_0 = R"doc()doc";


static const char* __doc_gr_branchless_quad_45deg_slicer_1 = R"doc()doc";


static const char* __doc_gr_p2_round_down = R"doc()doc";


static const char* __doc_gr_p2_round_up = R"doc()doc";


static const char* __doc_gr_p2_modulo = R"doc()doc";


static const char* __doc_gr_p2_modulo_neg = R"doc()doc";
