/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(add_const_ff.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(38f26cf5ffa1dccfded03ad4d67d7639)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/add_const_ff.h>
// pydoc.h is automatically generated in the build directory
#include <add_const_ff_pydoc.h>

void bind_add_const_ff(py::module& m)
{

    using add_const_ff = ::gr::blocks::add_const_ff;


    py::class_<add_const_ff,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<add_const_ff>>(m, "add_const_ff", D(add_const_ff))

        .def(py::init(&add_const_ff::make), py::arg("k"), D(add_const_ff, make))


        .def("k", &add_const_ff::k, D(add_const_ff, k))


        .def("set_k", &add_const_ff::set_k, py::arg("k"), D(add_const_ff, set_k))

        ;
}
