/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(feedforward_agc_cc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(6b9c22fa731776e36f0b2746d83822b3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/analog/feedforward_agc_cc.h>
// pydoc.h is automatically generated in the build directory
#include <feedforward_agc_cc_pydoc.h>

void bind_feedforward_agc_cc(py::module& m)
{

    using feedforward_agc_cc = ::gr::analog::feedforward_agc_cc;


    py::class_<feedforward_agc_cc,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<feedforward_agc_cc>>(
        m, "feedforward_agc_cc", D(feedforward_agc_cc))

        .def(py::init(&feedforward_agc_cc::make),
             py::arg("nsamples"),
             py::arg("reference"),
             D(feedforward_agc_cc, make))


        ;
}
