/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/messages/msg_queue.h>

void bind_messages_msg_queue(py::module& m)
{
    using msg_queue = gr::messages::msg_queue;


    py::class_<msg_queue, std::shared_ptr<msg_queue>>(m, "messages_msg_queue")

        .def(py::init<unsigned int>(), py::arg("limit"))

        .def("insert_tail", &msg_queue::insert_tail, py::arg("msg"))
        .def("delete_head", &msg_queue::delete_head)
        .def("delete_head_nowait", &msg_queue::delete_head_nowait)
        .def("flush", &msg_queue::flush)
        .def("empty_p", &msg_queue::empty_p)
        .def("full_p", &msg_queue::full_p)
        .def("count", &msg_queue::count)
        .def("limit", &msg_queue::limit);


    // m.def("make_msg_queue",&gr::messages::make_msg_queue,
    //     py::arg("limit") = 0
    // );
}
