/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(wavfile_source.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(01a042890c5fe1a8688deff99d957816)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/wavfile_source.h>
// pydoc.h is automatically generated in the build directory
#include <wavfile_source_pydoc.h>

void bind_wavfile_source(py::module& m)
{

    using wavfile_source = ::gr::blocks::wavfile_source;


    py::class_<wavfile_source,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<wavfile_source>>(m, "wavfile_source", D(wavfile_source))

        .def(py::init(&wavfile_source::make),
             py::arg("filename"),
             py::arg("repeat") = false,
             D(wavfile_source, make))


        .def("sample_rate", &wavfile_source::sample_rate, D(wavfile_source, sample_rate))


        .def("bits_per_sample",
             &wavfile_source::bits_per_sample,
             D(wavfile_source, bits_per_sample))


        .def("channels", &wavfile_source::channels, D(wavfile_source, channels))

        ;
}
