/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, qtgui, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_qtgui_ber_sink_b = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_qwidget = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_ber_sink_b_0 = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_ber_sink_b_1 = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_make = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_exec_ = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_y_axis = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_x_axis = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_update_time = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_title = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_label = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_color = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_width = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_style = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_marker = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_line_alpha = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_title = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_label = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_color = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_width = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_style = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_marker = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_line_alpha = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_set_size = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_enable_menu = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_enable_autoscale = R"doc()doc";


static const char* __doc_gr_qtgui_ber_sink_b_nsamps = R"doc()doc";
