/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(multiply_by_tag_value_cc.h) */
/* BINDTOOL_HEADER_FILE_HASH(83e16476793f2a5f6173d0bcd70cbf41)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/multiply_by_tag_value_cc.h>
// pydoc.h is automatically generated in the build directory
#include <multiply_by_tag_value_cc_pydoc.h>

void bind_multiply_by_tag_value_cc(py::module& m)
{

    using multiply_by_tag_value_cc = ::gr::blocks::multiply_by_tag_value_cc;


    py::class_<multiply_by_tag_value_cc,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<multiply_by_tag_value_cc>>(
        m, "multiply_by_tag_value_cc", D(multiply_by_tag_value_cc))

        .def(py::init(&multiply_by_tag_value_cc::make),
             py::arg("tag_name"),
             py::arg("vlen") = 1,
             D(multiply_by_tag_value_cc, make))


        .def("k", &multiply_by_tag_value_cc::k, D(multiply_by_tag_value_cc, k))

        ;
}
