/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(sink.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(656d21133902e8ba6177998cb10c59a3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/audio/sink.h>
// pydoc.h is automatically generated in the build directory
#include <sink_pydoc.h>

void bind_sink(py::module& m)
{

    using sink = ::gr::audio::sink;


    py::class_<sink, gr::sync_block, gr::block, gr::basic_block, std::shared_ptr<sink>>(
        m, "sink", D(sink))

        .def(py::init(&sink::make),
             py::arg("sampling_rate"),
             py::arg("device_name") = "",
             py::arg("ok_to_block") = true,
             D(sink, make))


        ;
}
