/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pm_remez.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(36288e30d78a39eb4d38cbbdde47dc0e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/pm_remez.h>
// pydoc.h is automatically generated in the build directory
#include <pm_remez_pydoc.h>

void bind_pm_remez(py::module& m)
{


    m.def("pm_remez",
          &::gr::filter::pm_remez,
          py::arg("order"),
          py::arg("bands"),
          py::arg("ampl"),
          py::arg("error_weight"),
          py::arg("filter_type") = "bandpass",
          py::arg("grid_density") = 16,
          D(pm_remez));
}
