/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, trellis, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_trellis_fsm = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_0 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_1 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_2 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_3 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_4 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_5 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_6 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_7 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_8 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_fsm_9 = R"doc()doc";


static const char* __doc_gr_trellis_fsm_I = R"doc()doc";


static const char* __doc_gr_trellis_fsm_S = R"doc()doc";


static const char* __doc_gr_trellis_fsm_O = R"doc()doc";


static const char* __doc_gr_trellis_fsm_NS = R"doc()doc";


static const char* __doc_gr_trellis_fsm_OS = R"doc()doc";


static const char* __doc_gr_trellis_fsm_PS = R"doc()doc";


static const char* __doc_gr_trellis_fsm_PI = R"doc()doc";


static const char* __doc_gr_trellis_fsm_TMi = R"doc()doc";


static const char* __doc_gr_trellis_fsm_TMl = R"doc()doc";


static const char* __doc_gr_trellis_fsm_write_trellis_svg = R"doc()doc";


static const char* __doc_gr_trellis_fsm_write_fsm_txt = R"doc()doc";
