/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(iir_filter_ffd.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(1ae1c4f65beee6f1388cd4babbe2607e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/iir_filter_ffd.h>
// pydoc.h is automatically generated in the build directory
#include <iir_filter_ffd_pydoc.h>

void bind_iir_filter_ffd(py::module& m)
{

    using iir_filter_ffd = ::gr::filter::iir_filter_ffd;


    py::class_<iir_filter_ffd,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<iir_filter_ffd>>(m, "iir_filter_ffd", D(iir_filter_ffd))

        .def(py::init(&iir_filter_ffd::make),
             py::arg("fftaps"),
             py::arg("fbtaps"),
             py::arg("oldstyle") = true,
             D(iir_filter_ffd, make))


        .def("set_taps",
             &iir_filter_ffd::set_taps,
             py::arg("fftaps"),
             py::arg("fbtaps"),
             D(iir_filter_ffd, set_taps))

        ;
}
