/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(single_pole_iir_filter_cc.h) */
/* BINDTOOL_HEADER_FILE_HASH(eb91fccc856160922658e568b981cdc3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/single_pole_iir_filter_cc.h>
// pydoc.h is automatically generated in the build directory
#include <single_pole_iir_filter_cc_pydoc.h>

void bind_single_pole_iir_filter_cc(py::module& m)
{

    using single_pole_iir_filter_cc = ::gr::filter::single_pole_iir_filter_cc;


    py::class_<single_pole_iir_filter_cc,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<single_pole_iir_filter_cc>>(
        m, "single_pole_iir_filter_cc", D(single_pole_iir_filter_cc))

        .def(py::init(&single_pole_iir_filter_cc::make),
             py::arg("alpha"),
             py::arg("vlen") = 1,
             D(single_pole_iir_filter_cc, make))


        .def("set_taps",
             &single_pole_iir_filter_cc::set_taps,
             py::arg("alpha"),
             D(single_pole_iir_filter_cc, set_taps))

        ;
}
