/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, fft, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_fft_window = R"doc()doc";


static const char* __doc_gr_fft_window_window_0 = R"doc()doc";


static const char* __doc_gr_fft_window_window_1 = R"doc()doc";


static const char* __doc_gr_fft_window_max_attenuation = R"doc()doc";


static const char* __doc_gr_fft_window_coswindow_0 = R"doc()doc";


static const char* __doc_gr_fft_window_coswindow_1 = R"doc()doc";


static const char* __doc_gr_fft_window_coswindow_2 = R"doc()doc";


static const char* __doc_gr_fft_window_rectangular = R"doc()doc";


static const char* __doc_gr_fft_window_hamming = R"doc()doc";


static const char* __doc_gr_fft_window_hann = R"doc()doc";


static const char* __doc_gr_fft_window_hanning = R"doc()doc";


static const char* __doc_gr_fft_window_blackman = R"doc()doc";


static const char* __doc_gr_fft_window_blackman2 = R"doc()doc";


static const char* __doc_gr_fft_window_blackman3 = R"doc()doc";


static const char* __doc_gr_fft_window_blackman4 = R"doc()doc";


static const char* __doc_gr_fft_window_blackman_harris = R"doc()doc";


static const char* __doc_gr_fft_window_blackmanharris = R"doc()doc";


static const char* __doc_gr_fft_window_nuttall = R"doc()doc";


static const char* __doc_gr_fft_window_blackman_nuttall = R"doc()doc";


static const char* __doc_gr_fft_window_nuttall_cfd = R"doc()doc";


static const char* __doc_gr_fft_window_flattop = R"doc()doc";


static const char* __doc_gr_fft_window_kaiser = R"doc()doc";


static const char* __doc_gr_fft_window_bartlett = R"doc()doc";


static const char* __doc_gr_fft_window_welch = R"doc()doc";


static const char* __doc_gr_fft_window_parzen = R"doc()doc";


static const char* __doc_gr_fft_window_exponential = R"doc()doc";


static const char* __doc_gr_fft_window_riemann = R"doc()doc";


static const char* __doc_gr_fft_window_tukey = R"doc()doc";


static const char* __doc_gr_fft_window_gaussian = R"doc()doc";


static const char* __doc_gr_fft_window_build = R"doc()doc";
