/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(header_payload_demux.h)                                    */
/* BINDTOOL_HEADER_FILE_HASH(9631a916d8364f3b7d8989119620675c)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/header_payload_demux.h>
// pydoc.h is automatically generated in the build directory
#include <header_payload_demux_pydoc.h>

void bind_header_payload_demux(py::module& m)
{

    using header_payload_demux = ::gr::digital::header_payload_demux;


    py::class_<header_payload_demux,
               gr::block,
               gr::basic_block,
               std::shared_ptr<header_payload_demux>>(
        m, "header_payload_demux", D(header_payload_demux))

        .def(py::init(&header_payload_demux::make),
             py::arg("header_len"),
             py::arg("items_per_symbol") = 1,
             py::arg("guard_interval") = 0,
             py::arg("length_tag_key") = "frame_len",
             py::arg("trigger_tag_key") = "",
             py::arg("output_symbols") = false,
             py::arg("itemsize") = sizeof(gr_complex),
             py::arg("timing_tag_key") = "",
             py::arg("samp_rate") = 1.,
             py::arg("special_tags") = std::vector<std::string>(),
             py::arg("header_padding") = 0,
             D(header_payload_demux, make));
}
