/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(time_delta.h)                                              */
/* BINDTOOL_HEADER_FILE_HASH(e4e3bff38b7d8f483563bbe107696e28)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/pdu/time_delta.h>
// pydoc.h is automatically generated in the build directory
#include <time_delta_pydoc.h>

void bind_time_delta(py::module& m)
{

    using time_delta = gr::pdu::time_delta;


    py::class_<time_delta, gr::block, gr::basic_block, std::shared_ptr<time_delta>>(
        m, "time_delta", D(time_delta))

        .def(py::init(&time_delta::make),
             py::arg("delta_key"),
             py::arg("time_key"),
             D(time_delta, make))


        ;
}
