/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(siso_type.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(9855f50a23fc6624c9ab52828baec66a)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/trellis/siso_type.h>
// pydoc.h is automatically generated in the build directory
#include <siso_type_pydoc.h>

void bind_siso_type(py::module& m)
{
    py::enum_<::gr::trellis::siso_type_t>(m, "siso_type_t")
        .value("TRELLIS_MIN_SUM", ::gr::trellis::TRELLIS_MIN_SUM)         // 200
        .value("TRELLIS_SUM_PRODUCT", ::gr::trellis::TRELLIS_SUM_PRODUCT) // 201
        .export_values();

    py::implicitly_convertible<int, ::gr::trellis::siso_type_t>();
}
