/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/vocoder/gsm_fr_encode_sp.h>
// pydoc.h is automatically generated in the build directory
#include <gsm_fr_encode_sp_pydoc.h>

void bind_gsm_fr_encode_sp(py::module& m)
{

    using gsm_fr_encode_sp = ::gr::vocoder::gsm_fr_encode_sp;


    py::class_<gsm_fr_encode_sp,
               gr::sync_decimator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<gsm_fr_encode_sp>>(
        m, "gsm_fr_encode_sp", D(gsm_fr_encode_sp))

        .def(py::init(&gsm_fr_encode_sp::make), D(gsm_fr_encode_sp, make))


        ;
}
