/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(cc_decoder.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(d047cf6f0d6e65a23b03f1e03ee09b0d)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/cc_decoder.h>
// pydoc.h is automatically generated in the build directory
#include <cc_decoder_pydoc.h>

void bind_cc_decoder(py::module& m)
{


    py::module m_code = m.def_submodule("code");

    using cc_decoder = ::gr::fec::code::cc_decoder;


    py::class_<cc_decoder, gr::fec::generic_decoder, std::shared_ptr<cc_decoder>>(
        m_code, "cc_decoder", D(code, cc_decoder))

        // unable to use init because incompatible pointer type is returned
        .def_static("make",
                    &cc_decoder::make,
                    py::arg("frame_size"),
                    py::arg("k"),
                    py::arg("rate"),
                    py::arg("polys"),
                    py::arg("start_state") = 0,
                    py::arg("end_state") = -1,
                    py::arg("mode") = ::_cc_mode_t::CC_STREAMING,
                    py::arg("padded") = false,
                    D(code, cc_decoder, make))


        .def("set_frame_size",
             &cc_decoder::set_frame_size,
             py::arg("frame_size"),
             D(code, cc_decoder, set_frame_size))


        .def("rate", &cc_decoder::rate, D(code, cc_decoder, rate))

        ;
}
