/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(buffer_type.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(daf4c66b477389a2a4a9fb2a94a752de)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/buffer_type.h>
// pydoc.h is automatically generated in the build directory
#include <buffer_type_pydoc.h>

// NOTE: buffer_type is really a typedef of const buffer_type_base& so
// buffer_type_base is used below because that's the type we really care about
void bind_buffer_type(py::module& m)
{

    using buffer_type_base = ::gr::buffer_type_base;


    py::class_<buffer_type_base>(m, "buffer_type_base", D(buffer_type_base))

        .def("name", &buffer_type_base::name, D(buffer_type_base, name));
}
