/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(attr_source.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ab772aba3f3e7d99189110d924d56248)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/iio/attr_source.h>
// pydoc.h is automatically generated in the build directory
#include <attr_source_pydoc.h>

void bind_attr_source(py::module& m)
{

    using attr_source = gr::iio::attr_source;


    py::class_<attr_source,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<attr_source>>(m, "attr_source", D(attr_source))

        .def(py::init(&attr_source::make),
             py::arg("uri"),
             py::arg("device"),
             py::arg("channel"),
             py::arg("attribute"),
             py::arg("update_interval_ms"),
             py::arg("samples_per_update"),
             py::arg("data_type"),
             py::arg("attr_type"),
             py::arg("output"),
             py::arg("address"),
             D(attr_source, make))

        ;
}
