/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(adaptive_algorithm_lms.h) */
/* BINDTOOL_HEADER_FILE_HASH(bf34ea7730349513a808de7f99e7744b)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/adaptive_algorithm_lms.h>
// pydoc.h is automatically generated in the build directory
#include <adaptive_algorithm_lms_pydoc.h>

void bind_adaptive_algorithm_lms(py::module& m)
{

    using adaptive_algorithm_lms = ::gr::digital::adaptive_algorithm_lms;


    py::class_<adaptive_algorithm_lms,
               gr::digital::adaptive_algorithm,
               std::shared_ptr<adaptive_algorithm_lms>>(
        m, "adaptive_algorithm_lms", D(adaptive_algorithm_lms))

        .def(py::init(&adaptive_algorithm_lms::make),
             py::arg("cons"),
             py::arg("step_size"),
             D(adaptive_algorithm_lms, make))


        .def("update_taps",
             &adaptive_algorithm_lms::update_taps,
             py::arg("taps"),
             py::arg("in"),
             py::arg("error"),
             py::arg("decision"),
             py::arg("num_taps"),
             D(adaptive_algorithm_lms, update_taps))


        .def("update_tap",
             &adaptive_algorithm_lms::update_tap,
             py::arg("tap"),
             py::arg("u_n"),
             py::arg("err"),
             py::arg("decision"),
             D(adaptive_algorithm_lms, update_tap))


        .def("initialize_taps",
             &adaptive_algorithm_lms::initialize_taps,
             py::arg("taps"),
             D(adaptive_algorithm_lms, initialize_taps))

        ;
}
