/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(depuncture_bb.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(a1711f618d99f02ec85fdf11f19e512c)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/depuncture_bb.h>
// pydoc.h is automatically generated in the build directory
#include <depuncture_bb_pydoc.h>

void bind_depuncture_bb(py::module& m)
{

    using depuncture_bb = ::gr::fec::depuncture_bb;


    py::class_<depuncture_bb, gr::block, gr::basic_block, std::shared_ptr<depuncture_bb>>(
        m, "depuncture_bb", D(depuncture_bb))

        .def(py::init(&depuncture_bb::make),
             py::arg("puncsize"),
             py::arg("puncpat"),
             py::arg("delay") = 0,
             py::arg("symbol") = 127,
             D(depuncture_bb, make))


        ;
}
