/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(ctrlport_probe2_c.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(97afe362bce3aff913d17babd0742a11)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/ctrlport_probe2_c.h>
// pydoc.h is automatically generated in the build directory
#include <ctrlport_probe2_c_pydoc.h>

void bind_ctrlport_probe2_c(py::module& m)
{

    using ctrlport_probe2_c = ::gr::blocks::ctrlport_probe2_c;


    py::class_<ctrlport_probe2_c,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<ctrlport_probe2_c>>(
        m, "ctrlport_probe2_c", D(ctrlport_probe2_c))

        .def(py::init(&ctrlport_probe2_c::make),
             py::arg("id"),
             py::arg("desc"),
             py::arg("len"),
             py::arg("disp_mask"),
             D(ctrlport_probe2_c, make))


        .def("get", &ctrlport_probe2_c::get, D(ctrlport_probe2_c, get))


        .def("set_length",
             &ctrlport_probe2_c::set_length,
             py::arg("len"),
             D(ctrlport_probe2_c, set_length))

        ;
}
