/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/messages/msg_accepter.h>

void bind_messages_msg_accepter(py::module& m)
{
    using msg_accepter = gr::messages::msg_accepter;


    py::class_<msg_accepter, std::shared_ptr<msg_accepter>>(m, "messages_msg_accepter")

        .def("post", &msg_accepter::post, py::arg("which_port"), py::arg("msg"));
}
