/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(complex_to_mag_squared.h) */
/* BINDTOOL_HEADER_FILE_HASH(bbc07c4648cb21825962ff680bf463b9)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/complex_to_mag_squared.h>
// pydoc.h is automatically generated in the build directory
#include <complex_to_mag_squared_pydoc.h>

void bind_complex_to_mag_squared(py::module& m)
{

    using complex_to_mag_squared = ::gr::blocks::complex_to_mag_squared;


    py::class_<complex_to_mag_squared,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<complex_to_mag_squared>>(
        m, "complex_to_mag_squared", D(complex_to_mag_squared))

        .def(py::init(&complex_to_mag_squared::make),
             py::arg("vlen") = 1,
             D(complex_to_mag_squared, make))


        ;
}
