/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(test_tag_variable_rate_ff.h) */
/* BINDTOOL_HEADER_FILE_HASH(1088dd79d0f93165dfc8e984b0963588)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/test_tag_variable_rate_ff.h>
// pydoc.h is automatically generated in the build directory
#include <test_tag_variable_rate_ff_pydoc.h>

void bind_test_tag_variable_rate_ff(py::module& m)
{

    using test_tag_variable_rate_ff = ::gr::blocks::test_tag_variable_rate_ff;


    py::class_<test_tag_variable_rate_ff,
               gr::block,
               gr::basic_block,
               std::shared_ptr<test_tag_variable_rate_ff>>(
        m, "test_tag_variable_rate_ff", D(test_tag_variable_rate_ff))

        .def(py::init(&test_tag_variable_rate_ff::make),
             py::arg("update_once") = false,
             py::arg("update_step") = 0.001,
             D(test_tag_variable_rate_ff, make))


        ;
}
