/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_block = R"doc()doc";


static const char* __doc_gr_block_history = R"doc()doc";


static const char* __doc_gr_block_set_history = R"doc()doc";


static const char* __doc_gr_block_declare_sample_delay_0 = R"doc()doc";


static const char* __doc_gr_block_declare_sample_delay_1 = R"doc()doc";


static const char* __doc_gr_block_sample_delay = R"doc()doc";


static const char* __doc_gr_block_fixed_rate = R"doc()doc";


static const char* __doc_gr_block_forecast = R"doc()doc";


static const char* __doc_gr_block_general_work = R"doc()doc";


static const char* __doc_gr_block_start = R"doc()doc";


static const char* __doc_gr_block_stop = R"doc()doc";


static const char* __doc_gr_block_set_output_multiple = R"doc()doc";


static const char* __doc_gr_block_output_multiple = R"doc()doc";


static const char* __doc_gr_block_output_multiple_set = R"doc()doc";


static const char* __doc_gr_block_set_alignment = R"doc()doc";


static const char* __doc_gr_block_alignment = R"doc()doc";


static const char* __doc_gr_block_set_unaligned = R"doc()doc";


static const char* __doc_gr_block_unaligned = R"doc()doc";


static const char* __doc_gr_block_set_is_unaligned = R"doc()doc";


static const char* __doc_gr_block_is_unaligned = R"doc()doc";


static const char* __doc_gr_block_consume = R"doc()doc";


static const char* __doc_gr_block_consume_each = R"doc()doc";


static const char* __doc_gr_block_produce = R"doc()doc";


static const char* __doc_gr_block_set_relative_rate_0 = R"doc()doc";


static const char* __doc_gr_block_set_inverse_relative_rate = R"doc()doc";


static const char* __doc_gr_block_set_relative_rate_1 = R"doc()doc";


static const char* __doc_gr_block_relative_rate = R"doc()doc";


static const char* __doc_gr_block_relative_rate_i = R"doc()doc";


static const char* __doc_gr_block_relative_rate_d = R"doc()doc";


static const char* __doc_gr_block_mp_relative_rate = R"doc()doc";


static const char* __doc_gr_block_fixed_rate_ninput_to_noutput = R"doc()doc";


static const char* __doc_gr_block_fixed_rate_noutput_to_ninput = R"doc()doc";


static const char* __doc_gr_block_nitems_read = R"doc()doc";


static const char* __doc_gr_block_nitems_written = R"doc()doc";


static const char* __doc_gr_block_tag_propagation_policy = R"doc()doc";


static const char* __doc_gr_block_set_tag_propagation_policy = R"doc()doc";


static const char* __doc_gr_block_min_noutput_items = R"doc()doc";


static const char* __doc_gr_block_set_min_noutput_items = R"doc()doc";


static const char* __doc_gr_block_max_noutput_items = R"doc()doc";


static const char* __doc_gr_block_set_max_noutput_items = R"doc()doc";


static const char* __doc_gr_block_unset_max_noutput_items = R"doc()doc";


static const char* __doc_gr_block_is_set_max_noutput_items = R"doc()doc";


static const char* __doc_gr_block_expand_minmax_buffer = R"doc()doc";


static const char* __doc_gr_block_max_output_buffer = R"doc()doc";


static const char* __doc_gr_block_set_max_output_buffer_0 = R"doc()doc";


static const char* __doc_gr_block_set_max_output_buffer_1 = R"doc()doc";


static const char* __doc_gr_block_min_output_buffer = R"doc()doc";


static const char* __doc_gr_block_set_min_output_buffer_0 = R"doc()doc";


static const char* __doc_gr_block_set_min_output_buffer_1 = R"doc()doc";


static const char* __doc_gr_block_set_blkd_input_timer_value = R"doc()doc";


static const char* __doc_gr_block_blkd_input_timer_value = R"doc()doc";


static const char* __doc_gr_block_pc_noutput_items = R"doc()doc";


static const char* __doc_gr_block_pc_noutput_items_avg = R"doc()doc";


static const char* __doc_gr_block_pc_noutput_items_var = R"doc()doc";


static const char* __doc_gr_block_pc_nproduced = R"doc()doc";


static const char* __doc_gr_block_pc_nproduced_avg = R"doc()doc";


static const char* __doc_gr_block_pc_nproduced_var = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_0 = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_avg_0 = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_var_0 = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_1 = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_avg_1 = R"doc()doc";


static const char* __doc_gr_block_pc_input_buffers_full_var_1 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_0 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_avg_0 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_var_0 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_1 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_avg_1 = R"doc()doc";


static const char* __doc_gr_block_pc_output_buffers_full_var_1 = R"doc()doc";


static const char* __doc_gr_block_pc_work_time = R"doc()doc";


static const char* __doc_gr_block_pc_work_time_avg = R"doc()doc";


static const char* __doc_gr_block_pc_work_time_var = R"doc()doc";


static const char* __doc_gr_block_pc_work_time_total = R"doc()doc";


static const char* __doc_gr_block_pc_throughput_avg = R"doc()doc";


static const char* __doc_gr_block_reset_perf_counters = R"doc()doc";


static const char* __doc_gr_block_setup_pc_rpc = R"doc()doc";


static const char* __doc_gr_block_is_pc_rpc_set = R"doc()doc";


static const char* __doc_gr_block_no_pc_rpc = R"doc()doc";


static const char* __doc_gr_block_set_processor_affinity = R"doc()doc";


static const char* __doc_gr_block_unset_processor_affinity = R"doc()doc";


static const char* __doc_gr_block_processor_affinity = R"doc()doc";


static const char* __doc_gr_block_active_thread_priority = R"doc()doc";


static const char* __doc_gr_block_thread_priority = R"doc()doc";


static const char* __doc_gr_block_set_thread_priority = R"doc()doc";


static const char* __doc_gr_block_update_rate = R"doc()doc";


static const char* __doc_gr_block_system_handler = R"doc()doc";


static const char* __doc_gr_block_set_log_level = R"doc()doc";


static const char* __doc_gr_block_log_level = R"doc()doc";


static const char* __doc_gr_block_finished = R"doc()doc";


static const char* __doc_gr_block_detail = R"doc()doc";


static const char* __doc_gr_block_set_detail = R"doc()doc";


static const char* __doc_gr_block_notify_msg_neighbors = R"doc()doc";


static const char* __doc_gr_block_clear_finished = R"doc()doc";


static const char* __doc_gr_block_identifier = R"doc()doc";


static const char* __doc_gr_cast_to_block_sptr = R"doc()doc";
