/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(tag_share.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ddf2d42d7c903a68eb83efabdda39d2d)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/tag_share.h>
// pydoc.h is automatically generated in the build directory
#include <tag_share_pydoc.h>

void bind_tag_share(py::module& m)
{

    using tag_share = ::gr::blocks::tag_share;


    py::class_<tag_share,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<tag_share>>(m, "tag_share", D(tag_share))

        .def(py::init(&tag_share::make),
             py::arg("sizeof_io_item"),
             py::arg("sizeof_share_item"),
             py::arg("vlen") = 1,
             D(tag_share, make))


        ;
}
