/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, channels, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_channels_dynamic_channel_model = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_dynamic_channel_model_0 =
    R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_dynamic_channel_model_1 =
    R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_make = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_samp_rate = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_sro_dev_std = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_sro_dev_max = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_cfo_dev_std = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_cfo_dev_max = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_noise_amp = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_doppler_freq = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_K = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_samp_rate = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_sro_dev_std = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_sro_dev_max = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_cfo_dev_std = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_cfo_dev_max = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_noise_amp = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_doppler_freq = R"doc()doc";


static const char* __doc_gr_channels_dynamic_channel_model_set_K = R"doc()doc";
