/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(rpcmanager.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(d6d7ea32b0beaf9b65d6301831351152)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/rpcmanager.h>

void bind_rpcmanager(py::module& m)
{
    py::class_<rpcmanager>(m, "rpcmanager")
        .def(py::init())
        .def("get", &rpcmanager::get)
        .def("register_booter", &rpcmanager::register_booter);
}
