/*
 * Copyright 2024 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, uhd, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */

static const char* __doc_gr_uhd_rfnoc_replay = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_rfnoc_replay_0 = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_rfnoc_replay_1 = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_make = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_record = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_record_restart = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_play = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_stop_playback = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_set_record_type = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_set_play_type = R"doc()doc";

static const char* __doc_gr_uhd_rfnoc_replay_issue_stream_cmd = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_port_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_offset_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_size_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_time_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_cmd_repeat_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_debug_port_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_mem_fullness_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_mem_size_key = R"doc()doc";

static const char* __doc_gr_uhd_replay_word_size_key = R"doc()doc";
