/* Reed-Solomon decoder
 * Copyright 2002 Phil Karn, KA9Q
 * May be used under the terms of the GNU General Public License (GPL)
 */

#define FIXED
#include "fixed.h"

#include "decode_rs.h"
#include "encode_rs.h"
