/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(dc_blocker_ff.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(db779eddf4e4c247425115523ca41472)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/dc_blocker_ff.h>
// pydoc.h is automatically generated in the build directory
#include <dc_blocker_ff_pydoc.h>

void bind_dc_blocker_ff(py::module& m)
{
    using dc_blocker_ff = gr::filter::dc_blocker_ff;


    py::class_<dc_blocker_ff,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<dc_blocker_ff>>(m, "dc_blocker_ff", D(dc_blocker_ff))

        .def(py::init(&dc_blocker_ff::make),
             py::arg("D"),
             py::arg("long_form") = true,
             D(dc_blocker_ff, make))


        .def("group_delay", &dc_blocker_ff::group_delay, D(dc_blocker_ff, group_delay))

        ;
}
