/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_hier_block2 = R"doc()doc";


static const char* __doc_gr_hier_block2_self = R"doc()doc";


static const char* __doc_gr_hier_block2_connect_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_connect_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_msg_connect_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_msg_connect_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_msg_disconnect_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_msg_disconnect_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_disconnect_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_disconnect_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_disconnect_all = R"doc()doc";


static const char* __doc_gr_hier_block2_lock = R"doc()doc";


static const char* __doc_gr_hier_block2_unlock = R"doc()doc";


static const char* __doc_gr_hier_block2_max_output_buffer = R"doc()doc";


static const char* __doc_gr_hier_block2_set_max_output_buffer_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_set_max_output_buffer_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_min_output_buffer = R"doc()doc";


static const char* __doc_gr_hier_block2_set_min_output_buffer_0 = R"doc()doc";


static const char* __doc_gr_hier_block2_set_min_output_buffer_1 = R"doc()doc";


static const char* __doc_gr_hier_block2_flatten = R"doc()doc";


static const char* __doc_gr_hier_block2_to_hier_block2 = R"doc()doc";


static const char* __doc_gr_hier_block2_has_msg_port = R"doc()doc";


static const char* __doc_gr_hier_block2_message_port_is_hier = R"doc()doc";


static const char* __doc_gr_hier_block2_message_port_is_hier_in = R"doc()doc";


static const char* __doc_gr_hier_block2_message_port_is_hier_out = R"doc()doc";


static const char* __doc_gr_hier_block2_message_port_register_hier_in = R"doc()doc";


static const char* __doc_gr_hier_block2_message_port_register_hier_out = R"doc()doc";


static const char* __doc_gr_hier_block2_set_processor_affinity = R"doc()doc";


static const char* __doc_gr_hier_block2_unset_processor_affinity = R"doc()doc";


static const char* __doc_gr_hier_block2_processor_affinity = R"doc()doc";


static const char* __doc_gr_hier_block2_set_log_level = R"doc()doc";


static const char* __doc_gr_hier_block2_log_level = R"doc()doc";


static const char* __doc_gr_hier_block2_all_min_output_buffer_p = R"doc()doc";


static const char* __doc_gr_hier_block2_all_max_output_buffer_p = R"doc()doc";


static const char* __doc_gr_make_hier_block2 = R"doc()doc";


static const char* __doc_gr_dot_graph = R"doc()doc";


static const char* __doc_gr_cast_to_hier_block2_sptr = R"doc()doc";
