/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(hdlc_framer_pb.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(b39076a4bb75c3076527db220789099d)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/hdlc_framer_pb.h>
// pydoc.h is automatically generated in the build directory
#include <hdlc_framer_pb_pydoc.h>

void bind_hdlc_framer_pb(py::module& m)
{

    using hdlc_framer_pb = ::gr::digital::hdlc_framer_pb;


    py::class_<hdlc_framer_pb,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<hdlc_framer_pb>>(m, "hdlc_framer_pb", D(hdlc_framer_pb))

        .def(py::init(&hdlc_framer_pb::make),
             py::arg("frame_tag_name"),
             D(hdlc_framer_pb, make))


        ;
}
