/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(cpfsk_bc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(e702ef3d26dedd94a467c55c4c3701bd)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/analog/cpfsk_bc.h>
// pydoc.h is automatically generated in the build directory
#include <cpfsk_bc_pydoc.h>

void bind_cpfsk_bc(py::module& m)
{

    using cpfsk_bc = ::gr::analog::cpfsk_bc;


    py::class_<cpfsk_bc,
               gr::sync_interpolator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<cpfsk_bc>>(m, "cpfsk_bc", D(cpfsk_bc))

        .def(py::init(&cpfsk_bc::make),
             py::arg("k"),
             py::arg("ampl"),
             py::arg("samples_per_sym"),
             D(cpfsk_bc, make))


        .def("set_amplitude",
             &cpfsk_bc::set_amplitude,
             py::arg("amplitude"),
             D(cpfsk_bc, set_amplitude))


        .def("amplitude", &cpfsk_bc::amplitude, D(cpfsk_bc, amplitude))


        .def("freq", &cpfsk_bc::freq, D(cpfsk_bc, freq))


        .def("phase", &cpfsk_bc::phase, D(cpfsk_bc, phase))

        ;
}
