/*
 * Copyright 2024 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(rpcserver_booter_base.h) */
/* BINDTOOL_HEADER_FILE_HASH(c50c14280bc1f2a66d7b9b2143de2680)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/rpcserver_booter_base.h>

void bind_rpcserver_booter_base(py::module& m)
{

    py::class_<rpcserver_booter_base>(m, "rpcserver_booter_base")
        // .def(py::init())
        // .def("i", &rpcserver_booter_base::i)
        .def("endpoints", &rpcserver_booter_base::endpoints)
        .def("type", &rpcserver_booter_base::type);
}