/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(1)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(udp_source.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(e8dceb0f73955c17260a5277ff77a7bc)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/network/udp_source.h>
// pydoc.h is automatically generated in the build directory
#include <udp_source_pydoc.h>

void bind_udp_source(py::module& m)
{

    using udp_source = ::gr::network::udp_source;


    py::class_<udp_source,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<udp_source>>(m, "udp_source", D(udp_source))

        .def(py::init(&udp_source::make),
             py::arg("itemsize"),
             py::arg("vecLen"),
             py::arg("port"),
             py::arg("header_type"),
             py::arg("payloadsize"),
             py::arg("notify_missed"),
             py::arg("source_zeros"),
             py::arg("ipv6"),
             D(udp_source, make))


        ;
}
