/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pack_k_bits.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(9417a28ce0ff8b790496cb1d33fca83f)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/pack_k_bits.h>
// pydoc.h is automatically generated in the build directory
#include <pack_k_bits_pydoc.h>

void bind_pack_k_bits(py::module& m)
{


    py::module m_kernel = m.def_submodule("kernel");

    using pack_k_bits = ::gr::blocks::kernel::pack_k_bits;


    py::class_<pack_k_bits, std::shared_ptr<pack_k_bits>>(
        m_kernel, "pack_k_bits", D(kernel, pack_k_bits))

        .def(py::init<unsigned int>(),
             py::arg("k"),
             D(kernel, pack_k_bits, pack_k_bits, 0))
        .def(py::init<gr::blocks::kernel::pack_k_bits const&>(),
             py::arg("arg0"),
             D(kernel, pack_k_bits, pack_k_bits, 1))


        .def("pack",
             &pack_k_bits::pack,
             py::arg("bytes"),
             py::arg("bits"),
             py::arg("nbytes"),
             D(kernel, pack_k_bits, pack))


        .def("pack_rev",
             &pack_k_bits::pack_rev,
             py::arg("bytes"),
             py::arg("bits"),
             py::arg("nbytes"),
             D(kernel, pack_k_bits, pack_rev))


        .def("k", &pack_k_bits::k, D(kernel, pack_k_bits, k))

        ;
}
