/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(pmt, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_pmt_mp_0 = R"doc()doc";


static const char* __doc_pmt_mp_1 = R"doc()doc";


static const char* __doc_pmt_mp_2 = R"doc()doc";


static const char* __doc_pmt_mp_3 = R"doc()doc";


static const char* __doc_pmt_mp_4 = R"doc()doc";


static const char* __doc_pmt_mp_5 = R"doc()doc";


static const char* __doc_pmt_mp_6 = R"doc()doc";


static const char* __doc_pmt_mp_7 = R"doc()doc";


static const char* __doc_pmt_mp_8 = R"doc()doc";


static const char* __doc_pmt_mp_9 = R"doc()doc";


static const char* __doc_pmt_mp_10 = R"doc()doc";


static const char* __doc_pmt_mp_11 = R"doc()doc";


static const char* __doc_pmt_mp_12 = R"doc()doc";


static const char* __doc_pmt_mp_13 = R"doc()doc";


static const char* __doc_pmt_mp_14 = R"doc()doc";


static const char* __doc_pmt_mp_15 = R"doc()doc";


static const char* __doc_pmt_mp_16 = R"doc()doc";


static const char* __doc_pmt_mp_17 = R"doc()doc";


static const char* __doc_pmt_mp_18 = R"doc()doc";


static const char* __doc_pmt_mp_19 = R"doc()doc";


static const char* __doc_pmt_mp_20 = R"doc()doc";
