/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(random_pdu.h)                                              */
/* BINDTOOL_HEADER_FILE_HASH(0ec9de7a89f30dd5a050547975e7c3d4)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/pdu/random_pdu.h>
// pydoc.h is automatically generated in the build directory
#include <random_pdu_pydoc.h>

void bind_random_pdu(py::module& m)
{

    using random_pdu = ::gr::pdu::random_pdu;


    py::class_<random_pdu, gr::block, gr::basic_block, std::shared_ptr<random_pdu>>(
        m, "random_pdu", D(random_pdu))

        .def(py::init(&random_pdu::make),
             py::arg("mintime"),
             py::arg("maxtime"),
             py::arg("byte_mask") = 255,
             py::arg("length_modulo") = 1,
             D(random_pdu, make))


        ;
}
