/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(annotator_raw.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(b7f1c3039dd27a3627908e88bb327aec)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/annotator_raw.h>
// pydoc.h is automatically generated in the build directory
#include <annotator_raw_pydoc.h>

void bind_annotator_raw(py::module& m)
{

    using annotator_raw = ::gr::blocks::annotator_raw;


    py::class_<annotator_raw,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<annotator_raw>>(m, "annotator_raw", D(annotator_raw))

        .def(py::init(&annotator_raw::make),
             py::arg("sizeof_stream_item"),
             D(annotator_raw, make))


        .def("add_tag",
             &annotator_raw::add_tag,
             py::arg("offset"),
             py::arg("key"),
             py::arg("val"),
             D(annotator_raw, add_tag))

        ;
}
