/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(sync_decimator.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(093d5d333e9d0c9d14c71cc5c469942b)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/sync_decimator.h>
// pydoc.h is automatically generated in the build directory
#include <sync_decimator_pydoc.h>

void bind_sync_decimator(py::module& m)
{

    using sync_decimator = ::gr::sync_decimator;


    py::class_<sync_decimator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<sync_decimator>>(m, "sync_decimator", D(sync_decimator))


        .def("decimation", &sync_decimator::decimation, D(sync_decimator, decimation))


        .def("set_decimation",
             &sync_decimator::set_decimation,
             py::arg("decimation"),
             D(sync_decimator, set_decimation))


        .def("general_work",
             &sync_decimator::general_work,
             py::arg("noutput_items"),
             py::arg("ninput_items"),
             py::arg("input_items"),
             py::arg("output_items"),
             D(sync_decimator, general_work))


        .def("fixed_rate_ninput_to_noutput",
             &sync_decimator::fixed_rate_ninput_to_noutput,
             py::arg("ninput"),
             D(sync_decimator, fixed_rate_ninput_to_noutput))


        .def("fixed_rate_noutput_to_ninput",
             &sync_decimator::fixed_rate_noutput_to_ninput,
             py::arg("noutput"),
             D(sync_decimator, fixed_rate_noutput_to_ninput))

        ;


    py::module m_messages = m.def_submodule("messages");


    py::module m_thread = m.def_submodule("thread");
}
