/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(puncture_ff.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(fe9165c64fe2337dab002c2078c2f3b3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/puncture_ff.h>
// pydoc.h is automatically generated in the build directory
#include <puncture_ff_pydoc.h>

void bind_puncture_ff(py::module& m)
{

    using puncture_ff = ::gr::fec::puncture_ff;


    py::class_<puncture_ff, gr::block, gr::basic_block, std::shared_ptr<puncture_ff>>(
        m, "puncture_ff", D(puncture_ff))

        .def(py::init(&puncture_ff::make),
             py::arg("puncsize"),
             py::arg("puncpat"),
             py::arg("delay"),
             D(puncture_ff, make))


        ;
}
