/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, qtgui, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_qtgui_waterfall_sink_c = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_waterfall_sink_c_0 = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_waterfall_sink_c_1 = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_make = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_exec_ = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_qwidget = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_clear_data = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_fft_size = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_fft_size = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_time_per_fft = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_fft_average = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_fft_average = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_fft_window = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_fft_window = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_frequency_range = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_intensity_range = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_update_time = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_title = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_time_title = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_line_label = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_line_alpha = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_color_map = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_title = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_line_label = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_line_alpha = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_color_map = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_set_size = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_auto_scale = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_min_intensity = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_max_intensity = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_enable_menu = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_enable_grid = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_disable_legend = R"doc()doc";


static const char* __doc_gr_qtgui_waterfall_sink_c_enable_axis_labels = R"doc()doc";
