/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(host_buffer_copy.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(e2466b836be2ee25d724242d545f4768)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/host_buffer_copy.h>
// pydoc.h is automatically generated in the build directory
#include <host_buffer_copy_pydoc.h>

void bind_host_buffer_copy(py::module& m)
{

    using host_buffer_copy = ::gr::blocks::host_buffer_copy;


    py::class_<host_buffer_copy,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<host_buffer_copy>>(
        m, "host_buffer_copy", D(host_buffer_copy))

        .def(py::init(&host_buffer_copy::make),
             py::arg("history"),
             D(host_buffer_copy, make))


        ;
}
