/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(file_descriptor_source.h) */
/* BINDTOOL_HEADER_FILE_HASH(f2d96fddb065808e7d03d8111e2c0e36)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/file_descriptor_source.h>
// pydoc.h is automatically generated in the build directory
#include <file_descriptor_source_pydoc.h>

void bind_file_descriptor_source(py::module& m)
{

    using file_descriptor_source = ::gr::blocks::file_descriptor_source;


    py::class_<file_descriptor_source,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<file_descriptor_source>>(
        m, "file_descriptor_source", D(file_descriptor_source))

        .def(py::init(&file_descriptor_source::make),
             py::arg("itemsize"),
             py::arg("fd"),
             py::arg("repeat") = false,
             D(file_descriptor_source, make))


        ;
}
