/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, digital, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_digital_constellation = R"doc()doc";


static const char* __doc_gr_digital_constellation_constellation_0 = R"doc()doc";


static const char* __doc_gr_digital_constellation_constellation_1 = R"doc()doc";


static const char* __doc_gr_digital_constellation_constellation_2 = R"doc()doc";


static const char* __doc_gr_digital_constellation_map_to_points = R"doc()doc";


static const char* __doc_gr_digital_constellation_map_to_points_v = R"doc()doc";


static const char* __doc_gr_digital_constellation_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_decision_maker_v = R"doc()doc";


static const char* __doc_gr_digital_constellation_decision_maker_pe = R"doc()doc";


static const char* __doc_gr_digital_constellation_calc_metric = R"doc()doc";


static const char* __doc_gr_digital_constellation_calc_euclidean_metric = R"doc()doc";


static const char* __doc_gr_digital_constellation_calc_hard_symbol_metric = R"doc()doc";


static const char* __doc_gr_digital_constellation_points = R"doc()doc";


static const char* __doc_gr_digital_constellation_s_points = R"doc()doc";


static const char* __doc_gr_digital_constellation_v_points = R"doc()doc";


static const char* __doc_gr_digital_constellation_apply_pre_diff_code = R"doc()doc";


static const char* __doc_gr_digital_constellation_set_pre_diff_code = R"doc()doc";


static const char* __doc_gr_digital_constellation_pre_diff_code = R"doc()doc";


static const char* __doc_gr_digital_constellation_rotational_symmetry = R"doc()doc";


static const char* __doc_gr_digital_constellation_dimensionality = R"doc()doc";


static const char* __doc_gr_digital_constellation_bits_per_symbol = R"doc()doc";


static const char* __doc_gr_digital_constellation_arity = R"doc()doc";


static const char* __doc_gr_digital_constellation_base = R"doc()doc";


static const char* __doc_gr_digital_constellation_as_pmt = R"doc()doc";


static const char* __doc_gr_digital_constellation_gen_soft_dec_lut = R"doc()doc";


static const char* __doc_gr_digital_constellation_calc_soft_dec = R"doc()doc";


static const char* __doc_gr_digital_constellation_set_soft_dec_lut = R"doc()doc";


static const char* __doc_gr_digital_constellation_set_npwr = R"doc()doc";


static const char* __doc_gr_digital_constellation_normalize = R"doc()doc";


static const char* __doc_gr_digital_constellation_has_soft_dec_lut = R"doc()doc";


static const char* __doc_gr_digital_constellation_soft_dec_lut = R"doc()doc";


static const char* __doc_gr_digital_constellation_soft_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_calcdist = R"doc()doc";


static const char* __doc_gr_digital_constellation_calcdist_constellation_calcdist =
    R"doc()doc";


static const char* __doc_gr_digital_constellation_calcdist_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_calcdist_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_sector = R"doc()doc";


static const char* __doc_gr_digital_constellation_sector_constellation_sector_0 =
    R"doc()doc";


static const char* __doc_gr_digital_constellation_sector_constellation_sector_1 =
    R"doc()doc";


static const char* __doc_gr_digital_constellation_sector_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_rect = R"doc()doc";


static const char* __doc_gr_digital_constellation_rect_constellation_rect = R"doc()doc";


static const char* __doc_gr_digital_constellation_rect_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_expl_rect = R"doc()doc";


static const char* __doc_gr_digital_constellation_expl_rect_constellation_expl_rect =
    R"doc()doc";


static const char* __doc_gr_digital_constellation_expl_rect_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_psk = R"doc()doc";


static const char* __doc_gr_digital_constellation_psk_constellation_psk = R"doc()doc";


static const char* __doc_gr_digital_constellation_psk_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_bpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_bpsk_constellation_bpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_bpsk_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_bpsk_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_qpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_qpsk_constellation_qpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_qpsk_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_qpsk_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_dqpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_dqpsk_constellation_dqpsk = R"doc()doc";


static const char* __doc_gr_digital_constellation_dqpsk_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_dqpsk_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_constellation_8psk = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_decision_maker = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_natural = R"doc()doc";


static const char*
    __doc_gr_digital_constellation_8psk_natural_constellation_8psk_natural = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_natural_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_8psk_natural_decision_maker =
    R"doc()doc";


static const char* __doc_gr_digital_constellation_16qam = R"doc()doc";


static const char* __doc_gr_digital_constellation_16qam_constellation_16qam = R"doc()doc";


static const char* __doc_gr_digital_constellation_16qam_make = R"doc()doc";


static const char* __doc_gr_digital_constellation_16qam_decision_maker = R"doc()doc";
