/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_basic_block = R"doc()doc";


static const char* __doc_gr_basic_block_message_subscribers = R"doc()doc";


static const char* __doc_gr_basic_block_unique_id = R"doc()doc";


static const char* __doc_gr_basic_block_symbolic_id = R"doc()doc";


static const char* __doc_gr_basic_block_name = R"doc()doc";


static const char* __doc_gr_basic_block_symbol_name = R"doc()doc";


static const char* __doc_gr_basic_block_identifier = R"doc()doc";


static const char* __doc_gr_basic_block_input_signature = R"doc()doc";


static const char* __doc_gr_basic_block_output_signature = R"doc()doc";


static const char* __doc_gr_basic_block_to_basic_block = R"doc()doc";


static const char* __doc_gr_basic_block_alias_set = R"doc()doc";


static const char* __doc_gr_basic_block_alias = R"doc()doc";


static const char* __doc_gr_basic_block_alias_pmt = R"doc()doc";


static const char* __doc_gr_basic_block_set_block_alias = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_register_in = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_register_out = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_pub = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_sub = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_unsub = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_is_hier = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_is_hier_in = R"doc()doc";


static const char* __doc_gr_basic_block_message_port_is_hier_out = R"doc()doc";


static const char* __doc_gr_basic_block_message_ports_in = R"doc()doc";


static const char* __doc_gr_basic_block_message_ports_out = R"doc()doc";


static const char* __doc_gr_basic_block__post = R"doc()doc";


static const char* __doc_gr_basic_block_empty_p_0 = R"doc()doc";


static const char* __doc_gr_basic_block_empty_p_1 = R"doc()doc";


static const char* __doc_gr_basic_block_empty_handled_p_0 = R"doc()doc";


static const char* __doc_gr_basic_block_empty_handled_p_1 = R"doc()doc";


static const char* __doc_gr_basic_block_nmsgs = R"doc()doc";


static const char* __doc_gr_basic_block_insert_tail = R"doc()doc";


static const char* __doc_gr_basic_block_delete_head_nowait = R"doc()doc";


static const char* __doc_gr_basic_block_get_iterator = R"doc()doc";


static const char* __doc_gr_basic_block_erase_msg = R"doc()doc";


static const char* __doc_gr_basic_block_has_msg_port = R"doc()doc";


static const char* __doc_gr_basic_block_get_msg_map = R"doc()doc";


static const char* __doc_gr_basic_block_add_rpc_variable = R"doc()doc";


static const char* __doc_gr_basic_block_setup_rpc = R"doc()doc";


static const char* __doc_gr_basic_block_is_rpc_set = R"doc()doc";


static const char* __doc_gr_basic_block_rpc_set = R"doc()doc";


static const char* __doc_gr_basic_block_check_topology = R"doc()doc";


static const char* __doc_gr_basic_block_set_processor_affinity = R"doc()doc";


static const char* __doc_gr_basic_block_unset_processor_affinity = R"doc()doc";


static const char* __doc_gr_basic_block_processor_affinity = R"doc()doc";


static const char* __doc_gr_basic_block_set_log_level = R"doc()doc";


static const char* __doc_gr_basic_block_log_level = R"doc()doc";


static const char* __doc_gr_basic_block_ncurrently_allocated = R"doc()doc";
