/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(float_to_short.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(847d10fddd3eb75d0cbe403a313295d4)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/float_to_short.h>
// pydoc.h is automatically generated in the build directory
#include <float_to_short_pydoc.h>

void bind_float_to_short(py::module& m)
{

    using float_to_short = ::gr::blocks::float_to_short;


    py::class_<float_to_short,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<float_to_short>>(m, "float_to_short", D(float_to_short))

        .def(py::init(&float_to_short::make),
             py::arg("vlen") = 1,
             py::arg("scale") = 1.,
             D(float_to_short, make))


        .def("scale", &float_to_short::scale, D(float_to_short, scale))


        .def("set_scale",
             &float_to_short::set_scale,
             py::arg("scale"),
             D(float_to_short, set_scale))

        ;
}
