/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pdu_split.h)                                               */
/* BINDTOOL_HEADER_FILE_HASH(badbfef0fff462fec05eb888d8d3b956)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/pdu/pdu_split.h>
// pydoc.h is automatically generated in the build directory
#include <pdu_split_pydoc.h>

void bind_pdu_split(py::module& m)
{

    using pdu_split = ::gr::pdu::pdu_split;


    py::class_<pdu_split, gr::block, gr::basic_block, std::shared_ptr<pdu_split>>(
        m, "pdu_split", D(pdu_split))

        .def(py::init(&pdu_split::make),
             py::arg("pass_empty_data") = false,
             D(pdu_split, make))


        ;
}
