/*
 * Copyright 2022 Marcus Müller
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_dictionary_logger_backend = R"doc()doc";
static const char* __doc_gr_dictionary_logger_backend_dictionary_logger_backend =
    R"doc()doc";
static const char* __doc_gr_dictionary_logger_backend_sink_it_ = R"doc()doc";
static const char* __doc_gr_dictionary_logger_backend_flush_ = R"doc()doc";
static const char* __doc_gr_dictionary_logger_backend_get_map = R"doc()doc";
