/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(filter_delay_fc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(318340909cbdad8f966383445fdec0f4)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/filter_delay_fc.h>
// pydoc.h is automatically generated in the build directory
#include <filter_delay_fc_pydoc.h>

void bind_filter_delay_fc(py::module& m)
{
    using filter_delay_fc = gr::filter::filter_delay_fc;


    py::class_<filter_delay_fc,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<filter_delay_fc>>(m, "filter_delay_fc", D(filter_delay_fc))

        .def(py::init(&filter_delay_fc::make), py::arg("taps"), D(filter_delay_fc, make));
}
