/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(stretch_ff.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(80078e377cab0382f47419eb56896eb7)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/stretch_ff.h>
// pydoc.h is automatically generated in the build directory
#include <stretch_ff_pydoc.h>

void bind_stretch_ff(py::module& m)
{

    using stretch_ff = ::gr::blocks::stretch_ff;


    py::class_<stretch_ff,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<stretch_ff>>(m, "stretch_ff", D(stretch_ff))

        .def(py::init(&stretch_ff::make),
             py::arg("lo"),
             py::arg("vlen") = 1,
             D(stretch_ff, make))


        .def("lo", &stretch_ff::lo, D(stretch_ff, lo))


        .def("set_lo", &stretch_ff::set_lo, py::arg("lo"), D(stretch_ff, set_lo))


        .def("vlen", &stretch_ff::vlen, D(stretch_ff, vlen))

        ;
}
