/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, digital, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_digital_pfb_clock_sync_ccf = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_pfb_clock_sync_ccf_0 = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_pfb_clock_sync_ccf_1 = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_make = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_update_gains = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_update_taps = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_taps = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_diff_taps = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_channel_taps = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_diff_channel_taps = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_taps_as_string = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_diff_taps_as_string = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_set_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_set_damping_factor = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_set_alpha = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_set_beta = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_set_max_rate_deviation =
    R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_loop_bandwidth = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_damping_factor = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_alpha = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_beta = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_clock_rate = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_error = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_rate = R"doc()doc";


static const char* __doc_gr_digital_pfb_clock_sync_ccf_phase = R"doc()doc";
