/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(goertzel_fc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(4bed76c1708616efa30a68810d684540)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fft/goertzel_fc.h>
// pydoc.h is automatically generated in the build directory
#include <goertzel_fc_pydoc.h>

void bind_goertzel_fc(py::module& m)
{
    using goertzel_fc = gr::fft::goertzel_fc;


    py::class_<goertzel_fc,
               gr::sync_decimator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<goertzel_fc>>(m, "goertzel_fc", D(goertzel_fc))

        .def(py::init(&goertzel_fc::make),
             py::arg("rate"),
             py::arg("len"),
             py::arg("freq"),
             D(goertzel_fc, make))


        .def(
            "set_freq", &goertzel_fc::set_freq, py::arg("freq"), D(goertzel_fc, set_freq))
        .def(
            "set_rate", &goertzel_fc::set_rate, py::arg("rate"), D(goertzel_fc, set_rate))


        .def("freq", &goertzel_fc::freq, D(goertzel_fc, freq))


        .def("rate", &goertzel_fc::rate, D(goertzel_fc, rate))

        ;
}
