/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, filter, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_filter_firdes = R"doc()doc";


static const char* __doc_gr_filter_firdes_firdes_0 = R"doc()doc";


static const char* __doc_gr_filter_firdes_firdes_1 = R"doc()doc";


static const char* __doc_gr_filter_firdes_window = R"doc()doc";


static const char* __doc_gr_filter_firdes_low_pass = R"doc()doc";


static const char* __doc_gr_filter_firdes_low_pass_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_high_pass = R"doc()doc";


static const char* __doc_gr_filter_firdes_high_pass_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_band_pass = R"doc()doc";


static const char* __doc_gr_filter_firdes_band_pass_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_complex_band_pass = R"doc()doc";


static const char* __doc_gr_filter_firdes_complex_band_pass_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_band_reject = R"doc()doc";


static const char* __doc_gr_filter_firdes_band_reject_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_complex_band_reject = R"doc()doc";


static const char* __doc_gr_filter_firdes_complex_band_reject_2 = R"doc()doc";


static const char* __doc_gr_filter_firdes_hilbert = R"doc()doc";


static const char* __doc_gr_filter_firdes_root_raised_cosine = R"doc()doc";


static const char* __doc_gr_filter_firdes_gaussian = R"doc()doc";
