/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_flowgraph = R"doc()doc";


static const char* __doc_gr_flowgraph_flowgraph = R"doc()doc";


static const char* __doc_gr_flowgraph_connect_0 = R"doc()doc";


static const char* __doc_gr_flowgraph_disconnect_0 = R"doc()doc";


static const char* __doc_gr_flowgraph_connect_1 = R"doc()doc";


static const char* __doc_gr_flowgraph_disconnect_1 = R"doc()doc";


static const char* __doc_gr_flowgraph_connect_2 = R"doc()doc";


static const char* __doc_gr_flowgraph_disconnect_2 = R"doc()doc";


static const char* __doc_gr_flowgraph_validate = R"doc()doc";


static const char* __doc_gr_flowgraph_clear = R"doc()doc";


static const char* __doc_gr_flowgraph_edges = R"doc()doc";


static const char* __doc_gr_flowgraph_msg_edges = R"doc()doc";


static const char* __doc_gr_flowgraph_calc_used_blocks = R"doc()doc";


static const char* __doc_gr_flowgraph_topological_sort = R"doc()doc";


static const char* __doc_gr_flowgraph_partition = R"doc()doc";


static const char* __doc_gr_endpoint = R"doc()doc";


static const char* __doc_gr_endpoint_endpoint_0 = R"doc()doc";


static const char* __doc_gr_endpoint_endpoint_1 = R"doc()doc";


static const char* __doc_gr_endpoint_endpoint_2 = R"doc()doc";


static const char* __doc_gr_endpoint_block = R"doc()doc";


static const char* __doc_gr_endpoint_port = R"doc()doc";


static const char* __doc_gr_endpoint_identifier = R"doc()doc";


static const char* __doc_gr_msg_endpoint = R"doc()doc";


static const char* __doc_gr_msg_endpoint_msg_endpoint_0 = R"doc()doc";


static const char* __doc_gr_msg_endpoint_msg_endpoint_1 = R"doc()doc";


static const char* __doc_gr_msg_endpoint_msg_endpoint_2 = R"doc()doc";


static const char* __doc_gr_msg_endpoint_block = R"doc()doc";


static const char* __doc_gr_msg_endpoint_port = R"doc()doc";


static const char* __doc_gr_msg_endpoint_is_hier = R"doc()doc";


static const char* __doc_gr_msg_endpoint_set_hier = R"doc()doc";


static const char* __doc_gr_msg_endpoint_identifier = R"doc()doc";


static const char* __doc_gr_edge = R"doc()doc";


static const char* __doc_gr_edge_edge_0 = R"doc()doc";


static const char* __doc_gr_edge_edge_1 = R"doc()doc";


static const char* __doc_gr_edge_edge_2 = R"doc()doc";


static const char* __doc_gr_edge_src = R"doc()doc";


static const char* __doc_gr_edge_dst = R"doc()doc";


static const char* __doc_gr_edge_identifier = R"doc()doc";


static const char* __doc_gr_msg_edge = R"doc()doc";


static const char* __doc_gr_msg_edge_msg_edge_0 = R"doc()doc";


static const char* __doc_gr_msg_edge_msg_edge_1 = R"doc()doc";


static const char* __doc_gr_msg_edge_msg_edge_2 = R"doc()doc";


static const char* __doc_gr_msg_edge_src = R"doc()doc";


static const char* __doc_gr_msg_edge_dst = R"doc()doc";


static const char* __doc_gr_msg_edge_identifier = R"doc()doc";


static const char* __doc_gr_make_flowgraph = R"doc()doc";


static const char* __doc_gr_dot_graph_fg = R"doc()doc";
