/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(protocol_formatter_async.h) */
/* BINDTOOL_HEADER_FILE_HASH(546735020786d2501d5edb3a1867a025)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/protocol_formatter_async.h>
// pydoc.h is automatically generated in the build directory
#include <protocol_formatter_async_pydoc.h>

void bind_protocol_formatter_async(py::module& m)
{

    using protocol_formatter_async = ::gr::digital::protocol_formatter_async;


    py::class_<protocol_formatter_async,
               gr::block,
               gr::basic_block,
               std::shared_ptr<protocol_formatter_async>>(
        m, "protocol_formatter_async", D(protocol_formatter_async))

        .def(py::init(&protocol_formatter_async::make),
             py::arg("format"),
             D(protocol_formatter_async, make))


        ;
}
