/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_logger = R"doc()doc";
static const char* __doc_gr_logger_logger = R"doc()doc";
static const char* __doc_gr_logger_set_level = R"doc()doc";
static const char* __doc_gr_logger_get_level = R"doc()doc";
static const char* __doc_gr_logger_get_string_level = R"doc()doc";
static const char* __doc_gr_logger_trace = R"doc()doc";
static const char* __doc_gr_logger_debug = R"doc()doc";
static const char* __doc_gr_logger_info = R"doc()doc";
static const char* __doc_gr_logger_notice = R"doc()doc";
static const char* __doc_gr_logger_warn = R"doc()doc";
static const char* __doc_gr_logger_error = R"doc()doc";
static const char* __doc_gr_logger_crit = R"doc()doc";
static const char* __doc_gr_logger_alert = R"doc()doc";
static const char* __doc_gr_logger_fatal = R"doc()doc";
static const char* __doc_gr_logger_emerg = R"doc()doc";

static const char* __doc_gr_logger_log = R"doc()doc";

static const char* __doc_gr_logging_singleton = R"doc()doc";
static const char* __doc_gr_logging_default_level = R"doc()doc";
static const char* __doc_gr_logging_set_default_level = R"doc()doc";
static const char* __doc_gr_logging_debug_level = R"doc()doc";
static const char* __doc_gr_logging_set_debug_level = R"doc()doc";
static const char* __doc_gr_logging_add_default_sink = R"doc()doc";
static const char* __doc_gr_logging_add_debug_sink = R"doc()doc";
static const char* __doc_gr_logging_add_default_console_sink = R"doc()doc";
static const char* __doc_gr_logging_add_debug_console_sink = R"doc()doc";
static const char* __doc_gr_logging_default_pattern = R"doc()doc";
