/*
 * Copyright 2021 Daniel Estevez <daniel@destevez.net>
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(diff_coding_type.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(d2a67d2eacf4d643b8df9d240a971837)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/diff_coding_type.h>
// pydoc.h is automatically generated in the build directory
#include <diff_coding_type_pydoc.h>

void bind_diff_coding_type(py::module& m)
{


    py::enum_<::gr::digital::diff_coding_type>(m, "diff_coding_type")
        .value("DIFF_DIFFERENTIAL", ::gr::digital::DIFF_DIFFERENTIAL) // 0
        .value("DIFF_NRZI", ::gr::digital::DIFF_NRZI)                 // 1
        .export_values();

    py::implicitly_convertible<int, ::gr::digital::diff_coding_type>();
}
