/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_xoroshiro128p_prng = R"doc()doc";


static const char* __doc_gr_xoroshiro128p_prng_xoroshiro128p_prng_0 = R"doc()doc";


static const char* __doc_gr_xoroshiro128p_prng_xoroshiro128p_prng_1 = R"doc()doc";


static const char* __doc_gr_xoroshiro128p_prng_min = R"doc()doc";


static const char* __doc_gr_xoroshiro128p_prng_max = R"doc()doc";


static const char* __doc_gr_xoroshiro128p_prng_seed = R"doc()doc";


static const char* __doc_gr_random = R"doc()doc";


static const char* __doc_gr_random_random_0 = R"doc()doc";


static const char* __doc_gr_random_random_1 = R"doc()doc";


static const char* __doc_gr_random_reseed = R"doc()doc";


static const char* __doc_gr_random_set_integer_limits = R"doc()doc";


static const char* __doc_gr_random_ran_int = R"doc()doc";


static const char* __doc_gr_random_ran1 = R"doc()doc";


static const char* __doc_gr_random_gasdev = R"doc()doc";


static const char* __doc_gr_random_laplacian = R"doc()doc";


static const char* __doc_gr_random_rayleigh = R"doc()doc";


static const char* __doc_gr_random_impulse = R"doc()doc";


static const char* __doc_gr_random_rayleigh_complex = R"doc()doc";
