/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(metric_type.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(bd60dc506d9405f531acebceaab4aec0)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/metric_type.h>
// pydoc.h is automatically generated in the build directory
#include <metric_type_pydoc.h>

void bind_metric_type(py::module& m)
{


    py::enum_<::gr::digital::trellis_metric_type_t>(m, "trellis_metric_type_t")
        .value("TRELLIS_EUCLIDEAN", ::gr::digital::TRELLIS_EUCLIDEAN)     // 200
        .value("TRELLIS_HARD_SYMBOL", ::gr::digital::TRELLIS_HARD_SYMBOL) // 201
        .value("TRELLIS_HARD_BIT", ::gr::digital::TRELLIS_HARD_BIT)       // 202
        .export_values();

    py::implicitly_convertible<int, ::gr::digital::trellis_metric_type_t>();
}
