/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(1)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(atsc_equalizer.h)                                          */
/* BINDTOOL_HEADER_FILE_HASH(33ffb39eb30666aadee1df49711f0fe1)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/dtv/atsc_equalizer.h>
// pydoc.h is automatically generated in the build directory
#include <atsc_equalizer_pydoc.h>

void bind_atsc_equalizer(py::module& m)
{

    using atsc_equalizer = ::gr::dtv::atsc_equalizer;


    py::class_<atsc_equalizer,
               gr::block,
               gr::basic_block,
               std::shared_ptr<atsc_equalizer>>(m, "atsc_equalizer", D(atsc_equalizer))

        .def(py::init(&atsc_equalizer::make), D(atsc_equalizer, make))


        .def("taps", &atsc_equalizer::taps, D(atsc_equalizer, taps))


        .def("data", &atsc_equalizer::data, D(atsc_equalizer, data))

        ;
}
