/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, filter, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_pfb_arb_resampler_ccf_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_pfb_arb_resampler_ccf_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_set_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_print_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_set_rate = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_set_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_taps_per_filter =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_interpolation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_decimation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_fractional_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_group_delay = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_phase_offset =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccf_filter = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_pfb_arb_resampler_ccc_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_pfb_arb_resampler_ccc_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_set_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_print_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_set_rate = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_set_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_taps_per_filter =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_interpolation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_decimation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_fractional_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_group_delay = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_phase_offset =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_ccc_filter = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_pfb_arb_resampler_fff_0 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_pfb_arb_resampler_fff_1 =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_set_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_print_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_set_rate = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_set_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_phase = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_taps_per_filter =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_interpolation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_decimation_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_fractional_rate =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_group_delay = R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_phase_offset =
    R"doc()doc";


static const char* __doc_gr_filter_kernel_pfb_arb_resampler_fff_filter = R"doc()doc";
