/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, filter, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     float,
                                                     std::complex<float>>
    _filter = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     float,
                                                     std::complex<float>>
    _filter_n = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     float,
                                                     std::complex<float>>
    _ntaps_ff = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     float,
                                                     std::complex<float>>
    _ntaps_fb = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     float,
                                                     std::complex<float>>
    _set_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     double,
                                                     std::complex<double>>
    _filter = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     double,
                                                     std::complex<double>>
    _filter_n = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     double,
                                                     std::complex<double>>
    _ntaps_ff = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     double,
                                                     std::complex<double>>
    _ntaps_fb = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     double,
                                                     std::complex<double>>
    _set_taps = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     std::complex<double>,
                                                     std::complex<double>>
    _filter = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     std::complex<double>,
                                                     std::complex<double>>
    _filter_n = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     std::complex<double>,
                                                     std::complex<double>>
    _ntaps_ff = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     std::complex<double>,
                                                     std::complex<double>>
    _ntaps_fb = R"doc()doc";


static const char* __doc_gr_filter_kernel_iir_filter<std::complex<float>,
                                                     std::complex<float>,
                                                     std::complex<double>,
                                                     std::complex<double>>
    _set_taps = R"doc()doc";
