/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(correctiq_auto.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(cf781a54a2ac7c62e333d6d6f6014562)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/correctiq_auto.h>
// pydoc.h is automatically generated in the build directory
#include <correctiq_auto_pydoc.h>

void bind_correctiq_auto(py::module& m)
{

    using correctiq_auto = ::gr::blocks::correctiq_auto;


    py::class_<correctiq_auto,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<correctiq_auto>>(m, "correctiq_auto", D(correctiq_auto))

        .def(py::init(&correctiq_auto::make),
             py::arg("samp_rate"),
             py::arg("freq"),
             py::arg("gain"),
             py::arg("sync_window"),
             D(correctiq_auto, make))


        .def("get_freq", &correctiq_auto::get_freq, D(correctiq_auto, get_freq))


        .def("get_gain", &correctiq_auto::get_gain, D(correctiq_auto, get_gain))


        .def("set_freq",
             &correctiq_auto::set_freq,
             py::arg("newValue"),
             D(correctiq_auto, set_freq))


        .def("set_gain",
             &correctiq_auto::set_gain,
             py::arg("newValue"),
             D(correctiq_auto, set_gain))


        ;
}
