/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pfb_synthesizer_ccf.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(8fcea75d75d2da92ab82de59d342715c)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/filter/pfb_synthesizer_ccf.h>
// pydoc.h is automatically generated in the build directory
#include <pfb_synthesizer_ccf_pydoc.h>

void bind_pfb_synthesizer_ccf(py::module& m)
{

    using pfb_synthesizer_ccf = ::gr::filter::pfb_synthesizer_ccf;


    py::class_<pfb_synthesizer_ccf,
               gr::sync_interpolator,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<pfb_synthesizer_ccf>>(
        m, "pfb_synthesizer_ccf", D(pfb_synthesizer_ccf))

        .def(py::init(&pfb_synthesizer_ccf::make),
             py::arg("numchans"),
             py::arg("taps"),
             py::arg("twox") = false,
             D(pfb_synthesizer_ccf, make))


        .def("set_taps",
             &pfb_synthesizer_ccf::set_taps,
             py::arg("taps"),
             D(pfb_synthesizer_ccf, set_taps))


        .def("print_taps",
             &pfb_synthesizer_ccf::print_taps,
             D(pfb_synthesizer_ccf, print_taps))


        .def("taps", &pfb_synthesizer_ccf::taps, D(pfb_synthesizer_ccf, taps))


        .def("set_channel_map",
             &pfb_synthesizer_ccf::set_channel_map,
             py::arg("map"),
             D(pfb_synthesizer_ccf, set_channel_map))


        .def("channel_map",
             &pfb_synthesizer_ccf::channel_map,
             D(pfb_synthesizer_ccf, channel_map))

        ;
}
