/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(constellation_receiver_cb.h) */
/* BINDTOOL_HEADER_FILE_HASH(42182631bf01257825f872ec7d4994ae)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/constellation_receiver_cb.h>
// pydoc.h is automatically generated in the build directory
#include <constellation_receiver_cb_pydoc.h>

void bind_constellation_receiver_cb(py::module& m)
{

    using constellation_receiver_cb = ::gr::digital::constellation_receiver_cb;


    py::class_<constellation_receiver_cb,
               gr::block,
               gr::basic_block,
               gr::blocks::control_loop,
               std::shared_ptr<constellation_receiver_cb>>(
        m, "constellation_receiver_cb", D(constellation_receiver_cb))

        .def(py::init(&constellation_receiver_cb::make),
             py::arg("constellation"),
             py::arg("loop_bw"),
             py::arg("fmin"),
             py::arg("fmax"),
             D(constellation_receiver_cb, make))


        .def("phase_error_tracking",
             &constellation_receiver_cb::phase_error_tracking,
             py::arg("phase_error"),
             D(constellation_receiver_cb, phase_error_tracking))

        ;
}
