/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(vector_map.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(9c88a0b04c0e86a66c4c57ca843f1cd3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/vector_map.h>
// pydoc.h is automatically generated in the build directory
#include <vector_map_pydoc.h>

void bind_vector_map(py::module& m)
{

    using vector_map = ::gr::blocks::vector_map;


    py::class_<vector_map,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<vector_map>>(m, "vector_map", D(vector_map))

        .def(py::init(&vector_map::make),
             py::arg("item_size"),
             py::arg("in_vlens"),
             py::arg("mapping"),
             D(vector_map, make))


        .def("set_mapping",
             &vector_map::set_mapping,
             py::arg("mapping"),
             D(vector_map, set_mapping))

        ;
}
