/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(abs_blk.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(56dfda2d14a2aa7945b7b44cd10aa8a0)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/abs_blk.h>

template <typename T>
void bind_abs_template(py::module& m, const char* classname)
{
    using abs_blk = gr::blocks::abs_blk<T>;

    py::class_<abs_blk,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<abs_blk>>(m, classname)
        .def(py::init(&gr::blocks::abs_blk<T>::make), py::arg("vlen") = 1);
}

void bind_abs_blk(py::module& m)
{
    bind_abs_template<std::int16_t>(m, "abs_ss");
    bind_abs_template<std::int32_t>(m, "abs_ii");
    bind_abs_template<float>(m, "abs_ff");
}
