#ifndef _MSC_UNISTD_H_ // [
#define _MSC_UNISTD_H_
#ifndef _MSC_VER // [
#error "Use this header only with Microsoft Visual C++ compilers!"
#endif // _MSC_VER ]

// Just for getpid in the vmcircbuf classes
// (at least as far as I can tell 2021)
#include <process.h>

#endif // _MSC_UNISTD_H_ ]
