/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(file_sink.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(7d4fa47d3509d715bbf1dd940304cadf)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/file_sink.h>
// pydoc.h is automatically generated in the build directory
#include <file_sink_pydoc.h>

void bind_file_sink(py::module& m)
{

    using file_sink = ::gr::blocks::file_sink;


    py::class_<file_sink,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               gr::blocks::file_sink_base,
               std::shared_ptr<file_sink>>(m, "file_sink", D(file_sink))

        .def(py::init(&file_sink::make),
             py::arg("itemsize"),
             py::arg("filename"),
             py::arg("append") = false,
             D(file_sink, make))


        ;
}
