/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_buffer = R"doc()doc";


static const char* __doc_gr_buffer_space_available = R"doc()doc";


static const char* __doc_gr_buffer_bufsize = R"doc()doc";


static const char* __doc_gr_buffer_base = R"doc()doc";


static const char* __doc_gr_buffer_write_pointer = R"doc()doc";


static const char* __doc_gr_buffer_update_write_pointer = R"doc()doc";


static const char* __doc_gr_buffer_set_done = R"doc()doc";


static const char* __doc_gr_buffer_done = R"doc()doc";


static const char* __doc_gr_buffer_link = R"doc()doc";


static const char* __doc_gr_buffer_nreaders = R"doc()doc";


static const char* __doc_gr_buffer_reader = R"doc()doc";


static const char* __doc_gr_buffer_mutex = R"doc()doc";


static const char* __doc_gr_buffer_nitems_written = R"doc()doc";


static const char* __doc_gr_buffer_reset_nitem_counter = R"doc()doc";


static const char* __doc_gr_buffer_get_sizeof_item = R"doc()doc";


static const char* __doc_gr_buffer_add_item_tag = R"doc()doc";


static const char* __doc_gr_buffer_remove_item_tag = R"doc()doc";


static const char* __doc_gr_buffer_prune_tags = R"doc()doc";


static const char* __doc_gr_buffer_get_tags_begin = R"doc()doc";


static const char* __doc_gr_buffer_get_tags_end = R"doc()doc";


static const char* __doc_gr_buffer_get_tags_lower_bound = R"doc()doc";


static const char* __doc_gr_buffer_get_tags_upper_bound = R"doc()doc";


//  static const char *__doc_gr_buffer_reader = R"doc()doc";


static const char* __doc_gr_buffer_reader_buffer_reader = R"doc()doc";


static const char* __doc_gr_buffer_reader_declare_sample_delay = R"doc()doc";


static const char* __doc_gr_buffer_reader_sample_delay = R"doc()doc";


static const char* __doc_gr_buffer_reader_items_available = R"doc()doc";


static const char* __doc_gr_buffer_reader_buffer = R"doc()doc";


static const char* __doc_gr_buffer_reader_max_possible_items_available = R"doc()doc";


static const char* __doc_gr_buffer_reader_read_pointer = R"doc()doc";


static const char* __doc_gr_buffer_reader_update_read_pointer = R"doc()doc";


static const char* __doc_gr_buffer_reader_set_done = R"doc()doc";


static const char* __doc_gr_buffer_reader_done = R"doc()doc";


static const char* __doc_gr_buffer_reader_mutex = R"doc()doc";


static const char* __doc_gr_buffer_reader_nitems_read = R"doc()doc";


static const char* __doc_gr_buffer_reader_reset_nitem_counter = R"doc()doc";


static const char* __doc_gr_buffer_reader_get_sizeof_item = R"doc()doc";


static const char* __doc_gr_buffer_reader_link = R"doc()doc";


static const char* __doc_gr_buffer_reader_get_tags_in_range = R"doc()doc";


static const char* __doc_gr_make_buffer = R"doc()doc";


static const char* __doc_gr_buffer_add_reader = R"doc()doc";


static const char* __doc_gr_buffer_ncurrently_allocated = R"doc()doc";


static const char* __doc_gr_buffer_reader_ncurrently_allocated = R"doc()doc";
