/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(burst_tagger.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(45216b99c8f33d8a28cfef305929ee40)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/burst_tagger.h>
// pydoc.h is automatically generated in the build directory
#include <burst_tagger_pydoc.h>

void bind_burst_tagger(py::module& m)
{

    using burst_tagger = ::gr::blocks::burst_tagger;


    py::class_<burst_tagger,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<burst_tagger>>(m, "burst_tagger", D(burst_tagger))

        .def(py::init(&burst_tagger::make), py::arg("itemsize"), D(burst_tagger, make))


        .def("set_true_tag",
             &burst_tagger::set_true_tag,
             py::arg("key"),
             py::arg("value"),
             D(burst_tagger, set_true_tag))


        .def("set_false_tag",
             &burst_tagger::set_false_tag,
             py::arg("key"),
             py::arg("value"),
             D(burst_tagger, set_false_tag))

        ;
}
