/*
 * Copyright 2020-2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, soapy, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_soapy_block = R"doc()doc";


static const char* __doc_gr_soapy_block_block_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_block_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_make = R"doc()doc";


static const char* __doc_gr_soapy_block_get_driver_key = R"doc()doc";


static const char* __doc_gr_soapy_block_get_hardware_key = R"doc()doc";


static const char* __doc_gr_soapy_block_get_hardware_info = R"doc()doc";


static const char* __doc_gr_soapy_block_set_frontend_mapping = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frontend_mapping = R"doc()doc";


static const char* __doc_gr_soapy_block_get_channel_info = R"doc()doc";


static const char* __doc_gr_soapy_block_set_sample_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_sample_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_sample_rate_range = R"doc()doc";


static const char* __doc_gr_soapy_block_set_frequency_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_set_frequency_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_list_frequencies = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_range_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_range_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_args_info = R"doc()doc";


static const char* __doc_gr_soapy_block_set_bandwidth = R"doc()doc";


static const char* __doc_gr_soapy_block_get_bandwidth = R"doc()doc";


static const char* __doc_gr_soapy_block_get_bandwidth_range = R"doc()doc";


static const char* __doc_gr_soapy_block_list_antennas = R"doc()doc";


static const char* __doc_gr_soapy_block_set_antenna = R"doc()doc";


static const char* __doc_gr_soapy_block_get_antenna = R"doc()doc";


static const char* __doc_gr_soapy_block_has_gain_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_set_gain_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_get_gain_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_list_gains = R"doc()doc";


static const char* __doc_gr_soapy_block_set_gain_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_set_gain_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_gain_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_gain_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_gain_range_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_gain_range_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_has_frequency_correction = R"doc()doc";


static const char* __doc_gr_soapy_block_set_frequency_correction = R"doc()doc";


static const char* __doc_gr_soapy_block_get_frequency_correction = R"doc()doc";


static const char* __doc_gr_soapy_block_has_dc_offset_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_set_dc_offset_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_get_dc_offset_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_has_dc_offset = R"doc()doc";


static const char* __doc_gr_soapy_block_set_dc_offset = R"doc()doc";


static const char* __doc_gr_soapy_block_get_dc_offset = R"doc()doc";


static const char* __doc_gr_soapy_block_has_iq_balance = R"doc()doc";


static const char* __doc_gr_soapy_block_set_iq_balance = R"doc()doc";


static const char* __doc_gr_soapy_block_get_iq_balance = R"doc()doc";


static const char* __doc_gr_soapy_block_has_iq_balance_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_set_iq_balance_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_get_iq_balance_mode = R"doc()doc";


static const char* __doc_gr_soapy_block_set_master_clock_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_master_clock_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_master_clock_rates = R"doc()doc";


static const char* __doc_gr_soapy_block_set_reference_clock_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_reference_clock_rate = R"doc()doc";


static const char* __doc_gr_soapy_block_get_reference_clock_rates = R"doc()doc";


static const char* __doc_gr_soapy_block_list_clock_sources = R"doc()doc";


static const char* __doc_gr_soapy_block_set_clock_source = R"doc()doc";


static const char* __doc_gr_soapy_block_get_clock_source = R"doc()doc";


static const char* __doc_gr_soapy_block_list_time_sources = R"doc()doc";


static const char* __doc_gr_soapy_block_set_time_source = R"doc()doc";


static const char* __doc_gr_soapy_block_get_time_source = R"doc()doc";


static const char* __doc_gr_soapy_block_has_hardware_time = R"doc()doc";


static const char* __doc_gr_soapy_block_get_hardware_time = R"doc()doc";


static const char* __doc_gr_soapy_block_set_hardware_time = R"doc()doc";


static const char* __doc_gr_soapy_block_list_sensors_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_sensor_info_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_sensor_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_list_sensors_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_sensor_info_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_sensor_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_list_register_interfaces = R"doc()doc";


static const char* __doc_gr_soapy_block_write_register = R"doc()doc";


static const char* __doc_gr_soapy_block_read_register = R"doc()doc";


static const char* __doc_gr_soapy_block_write_registers = R"doc()doc";


static const char* __doc_gr_soapy_block_read_registers = R"doc()doc";


static const char* __doc_gr_soapy_block_get_setting_info_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_write_setting_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_setting_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_get_setting_info_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_write_setting_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_setting_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_list_gpio_banks = R"doc()doc";


static const char* __doc_gr_soapy_block_write_gpio_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_write_gpio_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_gpio = R"doc()doc";


static const char* __doc_gr_soapy_block_write_gpio_dir_0 = R"doc()doc";


static const char* __doc_gr_soapy_block_write_gpio_dir_1 = R"doc()doc";


static const char* __doc_gr_soapy_block_read_gpio_dir = R"doc()doc";


static const char* __doc_gr_soapy_block_write_i2c = R"doc()doc";


static const char* __doc_gr_soapy_block_read_i2c = R"doc()doc";


static const char* __doc_gr_soapy_block_transact_spi = R"doc()doc";


static const char* __doc_gr_soapy_block_list_uarts = R"doc()doc";


static const char* __doc_gr_soapy_block_write_uart = R"doc()doc";


static const char* __doc_gr_soapy_block_read_uart = R"doc()doc";
