/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/messages/msg_producer.h>

void bind_messages_msg_producer(py::module& m)
{
    using msg_producer = gr::messages::msg_producer;


    py::class_<msg_producer, std::shared_ptr<msg_producer>>(m, "messages_msg_producer")

        .def("retrieve", &msg_producer::retrieve);
}
