/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pub_sink.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(0dc6073a224da6f95df9c04a2a507de3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/zeromq/pub_sink.h>
// pydoc.h is automatically generated in the build directory
#include <pub_sink_pydoc.h>

void bind_pub_sink(py::module& m)
{

    using pub_sink = ::gr::zeromq::pub_sink;


    py::class_<pub_sink,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<pub_sink>>(m, "pub_sink", D(pub_sink))

        .def(py::init(&pub_sink::make),
             py::arg("itemsize"),
             py::arg("vlen"),
             py::arg("address"),
             py::arg("timeout") = 100,
             py::arg("pass_tags") = false,
             py::arg("hwm") = -1,
             py::arg("key") = "",
             py::arg("drop_on_hwm") = true,
             py::arg("bind") = true,
             D(pub_sink, make))


        .def("last_endpoint", &pub_sink::last_endpoint, D(pub_sink, last_endpoint))

        ;
}
