/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_block_detail = R"doc()doc";


static const char* __doc_gr_block_detail_ninputs = R"doc()doc";


static const char* __doc_gr_block_detail_noutputs = R"doc()doc";


static const char* __doc_gr_block_detail_sink_p = R"doc()doc";


static const char* __doc_gr_block_detail_source_p = R"doc()doc";


static const char* __doc_gr_block_detail_set_done = R"doc()doc";


static const char* __doc_gr_block_detail_done = R"doc()doc";


static const char* __doc_gr_block_detail_set_input = R"doc()doc";


static const char* __doc_gr_block_detail_input = R"doc()doc";


static const char* __doc_gr_block_detail_set_output = R"doc()doc";


static const char* __doc_gr_block_detail_output = R"doc()doc";


static const char* __doc_gr_block_detail_consume = R"doc()doc";


static const char* __doc_gr_block_detail_consume_each = R"doc()doc";


static const char* __doc_gr_block_detail_produce = R"doc()doc";


static const char* __doc_gr_block_detail_produce_each = R"doc()doc";


static const char* __doc_gr_block_detail_nitems_read = R"doc()doc";


static const char* __doc_gr_block_detail_nitems_written = R"doc()doc";


static const char* __doc_gr_block_detail_reset_nitem_counters = R"doc()doc";


static const char* __doc_gr_block_detail_clear_tags = R"doc()doc";


static const char* __doc_gr_block_detail_add_item_tag = R"doc()doc";


static const char* __doc_gr_block_detail_remove_item_tag = R"doc()doc";


static const char* __doc_gr_block_detail_get_tags_in_range_0 = R"doc()doc";


static const char* __doc_gr_block_detail_get_tags_in_range_1 = R"doc()doc";


static const char* __doc_gr_block_detail_set_processor_affinity = R"doc()doc";


static const char* __doc_gr_block_detail_unset_processor_affinity = R"doc()doc";


static const char* __doc_gr_block_detail_thread_priority = R"doc()doc";


static const char* __doc_gr_block_detail_set_thread_priority = R"doc()doc";


static const char* __doc_gr_block_detail_start_perf_counters = R"doc()doc";


static const char* __doc_gr_block_detail_stop_perf_counters = R"doc()doc";


static const char* __doc_gr_block_detail_reset_perf_counters = R"doc()doc";


static const char* __doc_gr_block_detail_pc_noutput_items = R"doc()doc";


static const char* __doc_gr_block_detail_pc_nproduced = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_work_time = R"doc()doc";


static const char* __doc_gr_block_detail_pc_noutput_items_avg = R"doc()doc";


static const char* __doc_gr_block_detail_pc_nproduced_avg = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_avg_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_avg_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_avg_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_avg_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_work_time_avg = R"doc()doc";


static const char* __doc_gr_block_detail_pc_throughput_avg = R"doc()doc";


static const char* __doc_gr_block_detail_pc_noutput_items_var = R"doc()doc";


static const char* __doc_gr_block_detail_pc_nproduced_var = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_var_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_input_buffers_full_var_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_var_0 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_output_buffers_full_var_1 = R"doc()doc";


static const char* __doc_gr_block_detail_pc_work_time_var = R"doc()doc";


static const char* __doc_gr_block_detail_pc_work_time_total = R"doc()doc";


static const char* __doc_gr_block_detail_consumed = R"doc()doc";


static const char* __doc_gr_make_block_detail = R"doc()doc";


static const char* __doc_gr_block_detail_ncurrently_allocated = R"doc()doc";
