/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, qtgui, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_qtgui_number_sink = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_number_sink_0 = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_number_sink_1 = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_make = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_exec_ = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_qwidget = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_update_time = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_average = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_graph_type = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_color_0 = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_color_1 = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_label = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_min = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_max = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_title = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_unit = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_set_factor = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_average = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_graph_type = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_color_min = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_color_max = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_label = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_min = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_max = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_title = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_unit = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_factor = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_enable_menu = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_enable_autoscale = R"doc()doc";


static const char* __doc_gr_qtgui_number_sink_reset = R"doc()doc";
