/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, digital, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_digital_mpsk_snr_est = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_mpsk_snr_est_0 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_mpsk_snr_est_1 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_alpha = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_set_alpha = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_update = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_snr = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_signal = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_noise = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_simple = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_simple_mpsk_snr_est_simple_0 =
    R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_simple_mpsk_snr_est_simple_1 =
    R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_simple_update = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_simple_snr = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_skew = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_skew_mpsk_snr_est_skew_0 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_skew_mpsk_snr_est_skew_1 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_skew_update = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_skew_snr = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_m2m4 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_m2m4_mpsk_snr_est_m2m4_0 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_m2m4_mpsk_snr_est_m2m4_1 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_m2m4_update = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_m2m4_snr = R"doc()doc";


static const char* __doc_gr_digital_snr_est_m2m4 = R"doc()doc";


static const char* __doc_gr_digital_snr_est_m2m4_snr_est_m2m4_0 = R"doc()doc";


static const char* __doc_gr_digital_snr_est_m2m4_snr_est_m2m4_1 = R"doc()doc";


static const char* __doc_gr_digital_snr_est_m2m4_update = R"doc()doc";


static const char* __doc_gr_digital_snr_est_m2m4_snr = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_svr = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_svr_mpsk_snr_est_svr_0 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_svr_mpsk_snr_est_svr_1 = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_svr_update = R"doc()doc";


static const char* __doc_gr_digital_mpsk_snr_est_svr_snr = R"doc()doc";
