/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, analog, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_analog_pll_carriertracking_cc = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_pll_carriertracking_cc_0 =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_pll_carriertracking_cc_1 =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_make = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_lock_detector = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_squelch_enable = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_lock_threshold =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_loop_bandwidth =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_damping_factor =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_alpha = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_beta = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_frequency = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_phase = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_min_freq = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_set_max_freq = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_loop_bandwidth =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_damping_factor =
    R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_alpha = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_beta = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_frequency = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_phase = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_min_freq = R"doc()doc";


static const char* __doc_gr_analog_pll_carriertracking_cc_get_max_freq = R"doc()doc";
