/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/qtgui/trigger_mode.h>
// pydoc.h is automatically generated in the build directory
#include <trigger_mode_pydoc.h>

void bind_trigger_mode(py::module& m)
{
    py::enum_<::gr::qtgui::trigger_mode>(m, "trigger_mode")
        .value("TRIG_MODE_FREE", ::gr::qtgui::TRIG_MODE_FREE) // 0
        .value("TRIG_MODE_AUTO", ::gr::qtgui::TRIG_MODE_AUTO) // 1
        .value("TRIG_MODE_NORM", ::gr::qtgui::TRIG_MODE_NORM) // 2
        .value("TRIG_MODE_TAG", ::gr::qtgui::TRIG_MODE_TAG)   // 3
        .export_values();

    py::enum_<::gr::qtgui::trigger_slope>(m, "trigger_slope")
        .value("TRIG_SLOPE_POS", ::gr::qtgui::TRIG_SLOPE_POS) // 0
        .value("TRIG_SLOPE_NEG", ::gr::qtgui::TRIG_SLOPE_NEG) // 1
        .export_values();

    py::implicitly_convertible<int, ::gr::qtgui::trigger_mode>();
    py::implicitly_convertible<int, ::gr::qtgui::trigger_slope>();
}
