/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(add_system_time.h)                                         */
/* BINDTOOL_HEADER_FILE_HASH(577bac06b6db295b3384056d5dd8916d)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/pdu/add_system_time.h>
// pydoc.h is automatically generated in the build directory
#include <add_system_time_pydoc.h>

void bind_add_system_time(py::module& m)
{

    using add_system_time = gr::pdu::add_system_time;


    py::class_<add_system_time,
               gr::block,
               gr::basic_block,
               std::shared_ptr<add_system_time>>(m, "add_system_time", D(add_system_time))

        .def(py::init(&add_system_time::make), py::arg("key"), D(add_system_time, make))


        ;
}
