/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(endianness.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(1738ddb9b6c9db43eafe4b41583afec0)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/endianness.h>

void bind_endianness(py::module& m)
{

    py::enum_<gr::endianness_t>(m, "endianness_t")
        .value("GR_MSB_FIRST", gr::GR_MSB_FIRST) // 0
        .value("GR_LSB_FIRST", gr::GR_LSB_FIRST) // 1
        .export_values();
}
