/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(base.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(37fcf9b8740b0018f3cdd29abb0551de)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/trellis/base.h>
// pydoc.h is automatically generated in the build directory
#include <base_pydoc.h>

void bind_base(py::module& m)
{


    m.def("dec2base",
          &::gr::trellis::dec2base,
          py::arg("num"),
          py::arg("base"),
          py::arg("s"),
          D(dec2base));


    m.def("dec2bases",
          &::gr::trellis::dec2bases,
          py::arg("num"),
          py::arg("bases"),
          py::arg("s"),
          D(dec2bases));


    m.def(
        "base2dec", &::gr::trellis::base2dec, py::arg("s"), py::arg("base"), D(base2dec));


    m.def("bases2dec",
          &::gr::trellis::bases2dec,
          py::arg("s"),
          py::arg("bases"),
          D(bases2dec));
}
