/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pluto_utils.h)                                             */
/* BINDTOOL_HEADER_FILE_HASH(0955a4f40cd1fe794602d5e1af340c63)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/iio/pluto_utils.h>
// pydoc.h is automatically generated in the build directory
#include <pluto_utils_pydoc.h>

void bind_pluto_utils(py::module& m)
{

    m.def("get_pluto_uri", &gr::iio::get_pluto_uri, D(pluto_utils, get_pluto_uri));
}
