/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */
#include "pydoc_macros.h"
#define D(...) DOC(gr, __VA_ARGS__)
/*
  This file contains placeholders for docstrings for the Python bindings.
  Do not edit! These were automatically extracted during the binding process
  and will be overwritten during the build process
 */


static const char* __doc_gr_metadata_keys_pdu_num = R"doc()doc";


static const char* __doc_gr_metadata_keys_rx_time = R"doc()doc";


static const char* __doc_gr_metadata_keys_sample_rate = R"doc()doc";


static const char* __doc_gr_metadata_keys_sys_time = R"doc()doc";


static const char* __doc_gr_metadata_keys_tx_eob = R"doc()doc";


static const char* __doc_gr_metadata_keys_tx_time = R"doc()doc";


static const char* __doc_gr_metadata_keys_tx_sob = R"doc()doc";


static const char* __doc_gr_msgport_names_bpdu = R"doc()doc";


static const char* __doc_gr_msgport_names_conf = R"doc()doc";


static const char* __doc_gr_msgport_names_cpdu = R"doc()doc";


static const char* __doc_gr_msgport_names_detects = R"doc()doc";


static const char* __doc_gr_msgport_names_dict = R"doc()doc";


static const char* __doc_gr_msgport_names_fpdu = R"doc()doc";


static const char* __doc_gr_msgport_names_msg = R"doc()doc";


static const char* __doc_gr_msgport_names_pdu = R"doc()doc";


static const char* __doc_gr_msgport_names_pdus = R"doc()doc";


static const char* __doc_gr_msgport_names_vec = R"doc()doc";


static const char* __doc_gr_pdu_itemsize = R"doc()doc";


static const char* __doc_gr_pdu_type_matches = R"doc()doc";


static const char* __doc_gr_pdu_make_pdu_vector = R"doc()doc";


static const char* __doc_gr_pdu_type_from_pmt = R"doc()doc";
