/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/messages/msg_accepter_msgq.h>

void bind_messages_msg_accepter_msgq(py::module& m)
{
    using msg_accepter_msgq = gr::messages::msg_accepter_msgq;


    py::class_<msg_accepter_msgq, std::shared_ptr<msg_accepter_msgq>>(
        m, "messages_msg_accepter_msgq")

        .def(py::init<gr::messages::msg_queue_sptr>(), py::arg("msgq"))
        .def(py::init<gr::messages::msg_accepter_msgq const&>(), py::arg("arg0"))

        .def("post", &msg_accepter_msgq::post, py::arg("msg"))
        .def("msg_queue", &msg_accepter_msgq::msg_queue);
}
