/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(dds_control.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ff3fc63a928b75b6ed3e7bb82fd8cc71)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/iio/dds_control.h>
// pydoc.h is automatically generated in the build directory
#include <dds_control_pydoc.h>

void bind_dds_control(py::module& m)
{

    using dds_control = gr::iio::dds_control;


    py::class_<dds_control, gr::block, gr::basic_block, std::shared_ptr<dds_control>>(
        m, "dds_control", D(dds_control))

        .def(py::init(&dds_control::make),
             py::arg("uri"),
             py::arg("enabled"),
             py::arg("frequencies"),
             py::arg("phases"),
             py::arg("scales"),
             D(dds_control, make))

        .def("set_dds_confg",
             &dds_control::set_dds_confg,
             py::arg("frequencies"),
             py::arg("phases"),
             py::arg("scales"),
             D(dds_control, set_dds_confg))

        ;
}
