/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(conv_bit_corr_bb.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(103ae2f5c917b8a5598d732c1630f55c)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/conv_bit_corr_bb.h>
// pydoc.h is automatically generated in the build directory
#include <conv_bit_corr_bb_pydoc.h>

void bind_conv_bit_corr_bb(py::module& m)
{

    using conv_bit_corr_bb = ::gr::fec::conv_bit_corr_bb;


    py::class_<conv_bit_corr_bb,
               gr::block,
               gr::basic_block,
               std::shared_ptr<conv_bit_corr_bb>>(
        m, "conv_bit_corr_bb", D(conv_bit_corr_bb))

        .def(py::init(&conv_bit_corr_bb::make),
             py::arg("correlator"),
             py::arg("corr_sym"),
             py::arg("corr_len"),
             py::arg("cut"),
             py::arg("flush"),
             py::arg("thresh"),
             D(conv_bit_corr_bb, make))


        .def("data_garble_rate",
             &conv_bit_corr_bb::data_garble_rate,
             py::arg("taps"),
             py::arg("syn_density"),
             D(conv_bit_corr_bb, data_garble_rate))

        ;
}
