/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(stream_to_tagged_stream.h) */
/* BINDTOOL_HEADER_FILE_HASH(c3cbfe6e43e3ab12170dcc91a5d067df)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/blocks/stream_to_tagged_stream.h>
// pydoc.h is automatically generated in the build directory
#include <stream_to_tagged_stream_pydoc.h>

void bind_stream_to_tagged_stream(py::module& m)
{

    using stream_to_tagged_stream = ::gr::blocks::stream_to_tagged_stream;


    py::class_<stream_to_tagged_stream,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<stream_to_tagged_stream>>(
        m, "stream_to_tagged_stream", D(stream_to_tagged_stream))

        .def(py::init(&stream_to_tagged_stream::make),
             py::arg("itemsize"),
             py::arg("vlen"),
             py::arg("packet_len"),
             py::arg("len_tag_key"),
             D(stream_to_tagged_stream, make))


        .def("set_packet_len",
             &stream_to_tagged_stream::set_packet_len,
             py::arg("packet_len"),
             D(stream_to_tagged_stream, set_packet_len))


        .def("set_packet_len_pmt",
             &stream_to_tagged_stream::set_packet_len_pmt,
             py::arg("packet_len"),
             D(stream_to_tagged_stream, set_packet_len_pmt))

        ;
}
