/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(realtime.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(4fe8778231103a1a91b8032daec80655)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/realtime.h>
// pydoc.h is automatically generated in the build directory
#include <realtime_pydoc.h>

void bind_realtime(py::module& m)
{
    py::enum_<::gr::rt_status_t>(m, "rt_status_t")
        .value("RT_OK", ::gr::RT_OK)                           // 0
        .value("RT_NOT_IMPLEMENTED", ::gr::RT_NOT_IMPLEMENTED) // 1
        .value("RT_NO_PRIVS", ::gr::RT_NO_PRIVS)               // 2
        .value("RT_OTHER_ERROR", ::gr::RT_OTHER_ERROR)         // 3
        .export_values();
    py::enum_<::gr::rt_sched_policy>(m, "rt_sched_policy")
        .value("RT_SCHED_RR", ::gr::RT_SCHED_RR)     // 0
        .value("RT_SCHED_FIFO", ::gr::RT_SCHED_FIFO) // 1
        .export_values();

    m.def("enable_realtime_scheduling",
          &::gr::enable_realtime_scheduling,
          D(enable_realtime_scheduling));
}
