/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/cc_common.h>
// pydoc.h is automatically generated in the build directory
#include <cc_decoder_pydoc.h>

void bind_cc_common(py::module& m)
{
    py::enum_<::_cc_mode_t>(m, "_cc_mode_t")
        .value("CC_STREAMING", ::CC_STREAMING)   // 0
        .value("CC_TERMINATED", ::CC_TERMINATED) // 1
        .value("CC_TRUNCATED", ::CC_TRUNCATED)   // 2
        .value("CC_TAILBITING", ::CC_TAILBITING) // 3
        .export_values();

    py::implicitly_convertible<int, ::_cc_mode_t>();
}
