/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/* This file is automatically generated using bindtool */

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <pmt/pmt_sugar.h>
// pydoc.h is automatically generated in the build directory
#include <pmt_sugar_pydoc.h>

void bind_pmt_sugar(py::module& m)
{
    m.def("mp", (pmt::pmt_t(*)(std::string const&)) & ::pmt::mp, py::arg("s"), D(mp, 0));


    m.def("mp", (pmt::pmt_t(*)(char const*)) & ::pmt::mp, py::arg("s"), D(mp, 1));


    m.def("mp", (pmt::pmt_t(*)(long int)) & ::pmt::mp, py::arg("x"), D(mp, 2));


    m.def("mp", (pmt::pmt_t(*)(long unsigned int)) & ::pmt::mp, py::arg("x"), D(mp, 3));


    m.def("mp",
          (pmt::pmt_t(*)(long long unsigned int)) & ::pmt::mp,
          py::arg("x"),
          D(mp, 4));


    m.def("mp", (pmt::pmt_t(*)(int)) & ::pmt::mp, py::arg("x"), D(mp, 5));


    m.def("mp", (pmt::pmt_t(*)(double)) & ::pmt::mp, py::arg("x"), D(mp, 6));


    m.def(
        "mp", (pmt::pmt_t(*)(std::complex<double>)) & ::pmt::mp, py::arg("z"), D(mp, 7));


    m.def("mp", (pmt::pmt_t(*)(std::complex<float>)) & ::pmt::mp, py::arg("z"), D(mp, 8));


    //     m.def("mp",
    //           (pmt::pmt_t(*)(std::shared_ptr<gr::messages::msg_accepter>)) & ::pmt::mp,
    //           py::arg("ma"),
    //           D(mp, 9));


    m.def("mp",
          (pmt::pmt_t(*)(void const*, size_t)) & ::pmt::mp,
          py::arg("data"),
          py::arg("len_in_bytes"),
          D(mp, 10));


    m.def("mp", (pmt::pmt_t(*)(pmt::pmt_t const&)) & ::pmt::mp, py::arg("e0"), D(mp, 11));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&, pmt::pmt_t const&)) & ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          D(mp, 12));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&, pmt::pmt_t const&, pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          D(mp, 13));


    m.def(
        "mp",
        (pmt::pmt_t(*)(
            pmt::pmt_t const&, pmt::pmt_t const&, pmt::pmt_t const&, pmt::pmt_t const&)) &
            ::pmt::mp,
        py::arg("e0"),
        py::arg("e1"),
        py::arg("e2"),
        py::arg("e3"),
        D(mp, 14));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          D(mp, 15));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          py::arg("e5"),
          D(mp, 16));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          py::arg("e5"),
          py::arg("e6"),
          D(mp, 17));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          py::arg("e5"),
          py::arg("e6"),
          py::arg("e7"),
          D(mp, 18));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          py::arg("e5"),
          py::arg("e6"),
          py::arg("e7"),
          py::arg("e8"),
          D(mp, 19));


    m.def("mp",
          (pmt::pmt_t(*)(pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&,
                         pmt::pmt_t const&)) &
              ::pmt::mp,
          py::arg("e0"),
          py::arg("e1"),
          py::arg("e2"),
          py::arg("e3"),
          py::arg("e4"),
          py::arg("e5"),
          py::arg("e6"),
          py::arg("e7"),
          py::arg("e8"),
          py::arg("e9"),
          D(mp, 20));
}
