/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(ctrlport_probe_psd.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(c459d4fb223fee8425848ca9a45d401b)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fft/ctrlport_probe_psd.h>
// pydoc.h is automatically generated in the build directory
#include <ctrlport_probe_psd_pydoc.h>

void bind_ctrlport_probe_psd(py::module& m)
{
    using ctrlport_probe_psd = gr::fft::ctrlport_probe_psd;

    py::class_<ctrlport_probe_psd,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<ctrlport_probe_psd>>(
        m, "ctrlport_probe_psd", D(ctrlport_probe_psd))

        .def(py::init(&ctrlport_probe_psd::make),
             py::arg("id"),
             py::arg("desc"),
             py::arg("len"),
             D(ctrlport_probe_psd, make))


        .def("get", &ctrlport_probe_psd::get, D(ctrlport_probe_psd, get))

        .def("set_length",
             &ctrlport_probe_psd::set_length,
             py::arg("len"),
             D(ctrlport_probe_psd, set_length));
}
