/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(protocol_parser_b.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(44f5291426b28e2a41da9758fbc80772)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/digital/protocol_parser_b.h>
// pydoc.h is automatically generated in the build directory
#include <protocol_parser_b_pydoc.h>

void bind_protocol_parser_b(py::module& m)
{

    using protocol_parser_b = ::gr::digital::protocol_parser_b;


    py::class_<protocol_parser_b,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<protocol_parser_b>>(
        m, "protocol_parser_b", D(protocol_parser_b))

        .def(py::init(&protocol_parser_b::make),
             py::arg("format"),
             D(protocol_parser_b, make))


        ;
}
