/*
 * Copyright 2020 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(ldpc_encoder.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ad646e6a6d64e525a93a42118767ba23)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/fec/ldpc_encoder.h>
// pydoc.h is automatically generated in the build directory
#include <ldpc_encoder_pydoc.h>

void bind_ldpc_encoder(py::module& m)
{

    using ldpc_encoder = ::gr::fec::ldpc_encoder;


    py::class_<ldpc_encoder, gr::fec::generic_encoder, std::shared_ptr<ldpc_encoder>>(
        m, "ldpc_encoder", D(ldpc_encoder))

        .def_static(
            "make", &ldpc_encoder::make, py::arg("alist_file"), D(ldpc_encoder, make))


        .def("rate", &ldpc_encoder::rate, D(ldpc_encoder, rate))


        .def("set_frame_size",
             &ldpc_encoder::set_frame_size,
             py::arg("frame_size"),
             D(ldpc_encoder, set_frame_size))


        .def("get_output_size",
             &ldpc_encoder::get_output_size,
             D(ldpc_encoder, get_output_size))


        .def("get_input_size",
             &ldpc_encoder::get_input_size,
             D(ldpc_encoder, get_input_size))

        ;
}
