/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pdu_set.h)                                                 */
/* BINDTOOL_HEADER_FILE_HASH(c7fba803e84070656e7898eb601df1f3)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <gnuradio/pdu/pdu_set.h>
// pydoc.h is automatically generated in the build directory
#include <pdu_set_pydoc.h>

void bind_pdu_set(py::module& m)
{

    using pdu_set = ::gr::pdu::pdu_set;


    py::class_<pdu_set, gr::block, gr::basic_block, std::shared_ptr<pdu_set>>(
        m, "pdu_set", D(pdu_set))

        .def(py::init(&pdu_set::make), py::arg("k"), py::arg("v"), D(pdu_set, make))


        .def("set_key", &pdu_set::set_key, py::arg("key"), D(pdu_set, set_key))


        .def("set_val", &pdu_set::set_val, py::arg("val"), D(pdu_set, set_val))

        ;
}
